# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'revpicommander.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_win_revpicommander(object):
    def setupUi(self, win_revpicommander):
        win_revpicommander.setObjectName("win_revpicommander")
        win_revpicommander.resize(353, 299)
        win_revpicommander.setWindowTitle("RevPi Commander")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/main/ico/revpipycontrol.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        win_revpicommander.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(win_revpicommander)
        self.centralwidget.setObjectName("centralwidget")
        self.gl = QtWidgets.QVBoxLayout(self.centralwidget)
        self.gl.setObjectName("gl")
        self.hzl_connection = QtWidgets.QHBoxLayout()
        self.hzl_connection.setObjectName("hzl_connection")
        self.txt_host = QtWidgets.QLineEdit(self.centralwidget)
        self.txt_host.setFocusPolicy(QtCore.Qt.NoFocus)
        self.txt_host.setText("")
        self.txt_host.setReadOnly(True)
        self.txt_host.setObjectName("txt_host")
        self.hzl_connection.addWidget(self.txt_host)
        self.txt_connection = QtWidgets.QLineEdit(self.centralwidget)
        self.txt_connection.setFocusPolicy(QtCore.Qt.NoFocus)
        self.txt_connection.setText("")
        self.txt_connection.setReadOnly(True)
        self.txt_connection.setObjectName("txt_connection")
        self.hzl_connection.addWidget(self.txt_connection)
        self.gl.addLayout(self.hzl_connection)
        self.btn_plc_start = QtWidgets.QPushButton(self.centralwidget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/action/ico/system-run.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_plc_start.setIcon(icon1)
        self.btn_plc_start.setObjectName("btn_plc_start")
        self.gl.addWidget(self.btn_plc_start)
        self.btn_plc_stop = QtWidgets.QPushButton(self.centralwidget)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/action/ico/process-stop.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_plc_stop.setIcon(icon2)
        self.btn_plc_stop.setObjectName("btn_plc_stop")
        self.gl.addWidget(self.btn_plc_stop)
        self.btn_plc_restart = QtWidgets.QPushButton(self.centralwidget)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/action/ico/view-refresh.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_plc_restart.setIcon(icon3)
        self.btn_plc_restart.setObjectName("btn_plc_restart")
        self.gl.addWidget(self.btn_plc_restart)
        self.btn_plc_logs = QtWidgets.QPushButton(self.centralwidget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/action/ico/applications-utilities.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_plc_logs.setIcon(icon4)
        self.btn_plc_logs.setObjectName("btn_plc_logs")
        self.gl.addWidget(self.btn_plc_logs)
        self.hzl_status = QtWidgets.QHBoxLayout()
        self.hzl_status.setObjectName("hzl_status")
        self.lbl_status = QtWidgets.QLabel(self.centralwidget)
        self.lbl_status.setObjectName("lbl_status")
        self.hzl_status.addWidget(self.lbl_status)
        self.txt_status = QtWidgets.QLineEdit(self.centralwidget)
        self.txt_status.setFocusPolicy(QtCore.Qt.NoFocus)
        self.txt_status.setText("")
        self.txt_status.setAlignment(QtCore.Qt.AlignCenter)
        self.txt_status.setReadOnly(True)
        self.txt_status.setObjectName("txt_status")
        self.hzl_status.addWidget(self.txt_status)
        self.gl.addLayout(self.hzl_status)
        self.btn_plc_debug = QtWidgets.QPushButton(self.centralwidget)
        self.btn_plc_debug.setMinimumSize(QtCore.QSize(300, 0))
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/action/ico/edit-find.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_plc_debug.setIcon(icon5)
        self.btn_plc_debug.setCheckable(True)
        self.btn_plc_debug.setObjectName("btn_plc_debug")
        self.gl.addWidget(self.btn_plc_debug)
        win_revpicommander.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(win_revpicommander)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 353, 24))
        self.menubar.setObjectName("menubar")
        self.men_file = QtWidgets.QMenu(self.menubar)
        self.men_file.setObjectName("men_file")
        self.men_help = QtWidgets.QMenu(self.menubar)
        self.men_help.setObjectName("men_help")
        self.men_plc = QtWidgets.QMenu(self.menubar)
        self.men_plc.setObjectName("men_plc")
        self.men_connections = QtWidgets.QMenu(self.menubar)
        self.men_connections.setObjectName("men_connections")
        win_revpicommander.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(win_revpicommander)
        self.statusbar.setSizeGripEnabled(False)
        self.statusbar.setObjectName("statusbar")
        win_revpicommander.setStatusBar(self.statusbar)
        self.act_connections = QtWidgets.QAction(win_revpicommander)
        self.act_connections.setShortcut("Ctrl+N")
        self.act_connections.setObjectName("act_connections")
        self.act_search = QtWidgets.QAction(win_revpicommander)
        self.act_search.setShortcut("Ctrl+F")
        self.act_search.setObjectName("act_search")
        self.act_quit = QtWidgets.QAction(win_revpicommander)
        self.act_quit.setObjectName("act_quit")
        self.act_webpage = QtWidgets.QAction(win_revpicommander)
        self.act_webpage.setObjectName("act_webpage")
        self.act_info = QtWidgets.QAction(win_revpicommander)
        self.act_info.setObjectName("act_info")
        self.act_logs = QtWidgets.QAction(win_revpicommander)
        self.act_logs.setShortcut("Ctrl+L")
        self.act_logs.setObjectName("act_logs")
        self.act_options = QtWidgets.QAction(win_revpicommander)
        self.act_options.setShortcut("Ctrl+O")
        self.act_options.setObjectName("act_options")
        self.act_program = QtWidgets.QAction(win_revpicommander)
        self.act_program.setShortcut("Ctrl+P")
        self.act_program.setObjectName("act_program")
        self.act_developer = QtWidgets.QAction(win_revpicommander)
        self.act_developer.setShortcut("Ctrl+D")
        self.act_developer.setObjectName("act_developer")
        self.act_pictory = QtWidgets.QAction(win_revpicommander)
        self.act_pictory.setObjectName("act_pictory")
        self.act_disconnect = QtWidgets.QAction(win_revpicommander)
        self.act_disconnect.setShortcut("Ctrl+X")
        self.act_disconnect.setObjectName("act_disconnect")
        self.act_reset = QtWidgets.QAction(win_revpicommander)
        self.act_reset.setObjectName("act_reset")
        self.act_simulator = QtWidgets.QAction(win_revpicommander)
        self.act_simulator.setObjectName("act_simulator")
        self.men_file.addAction(self.act_connections)
        self.men_file.addAction(self.act_search)
        self.men_file.addSeparator()
        self.men_file.addAction(self.act_simulator)
        self.men_file.addSeparator()
        self.men_file.addAction(self.act_quit)
        self.men_help.addAction(self.act_webpage)
        self.men_help.addSeparator()
        self.men_help.addAction(self.act_info)
        self.men_plc.addAction(self.act_logs)
        self.men_plc.addAction(self.act_options)
        self.men_plc.addAction(self.act_program)
        self.men_plc.addAction(self.act_developer)
        self.men_plc.addSeparator()
        self.men_plc.addAction(self.act_pictory)
        self.men_plc.addAction(self.act_reset)
        self.men_plc.addSeparator()
        self.men_plc.addAction(self.act_disconnect)
        self.menubar.addAction(self.men_file.menuAction())
        self.menubar.addAction(self.men_plc.menuAction())
        self.menubar.addAction(self.men_connections.menuAction())
        self.menubar.addAction(self.men_help.menuAction())

        self.retranslateUi(win_revpicommander)
        self.act_quit.triggered.connect(win_revpicommander.close) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(win_revpicommander)
        win_revpicommander.setTabOrder(self.btn_plc_start, self.btn_plc_stop)
        win_revpicommander.setTabOrder(self.btn_plc_stop, self.btn_plc_restart)
        win_revpicommander.setTabOrder(self.btn_plc_restart, self.btn_plc_logs)
        win_revpicommander.setTabOrder(self.btn_plc_logs, self.btn_plc_debug)

    def retranslateUi(self, win_revpicommander):
        _translate = QtCore.QCoreApplication.translate
        self.btn_plc_start.setText(_translate("win_revpicommander", "PLC &start"))
        self.btn_plc_stop.setText(_translate("win_revpicommander", "PLC s&top"))
        self.btn_plc_restart.setText(_translate("win_revpicommander", "PLC restart"))
        self.btn_plc_logs.setText(_translate("win_revpicommander", "PLC &logs"))
        self.lbl_status.setText(_translate("win_revpicommander", "Status:"))
        self.btn_plc_debug.setText(_translate("win_revpicommander", "PLC watch &mode"))
        self.men_file.setTitle(_translate("win_revpicommander", "&File"))
        self.men_help.setTitle(_translate("win_revpicommander", "&Help"))
        self.men_plc.setTitle(_translate("win_revpicommander", "&PLC"))
        self.men_connections.setTitle(_translate("win_revpicommander", "&Connections"))
        self.act_connections.setText(_translate("win_revpicommander", "&Connections..."))
        self.act_search.setText(_translate("win_revpicommander", "&Search Revolution Pi..."))
        self.act_quit.setText(_translate("win_revpicommander", "&Quit"))
        self.act_webpage.setText(_translate("win_revpicommander", "Visit &webpage..."))
        self.act_info.setText(_translate("win_revpicommander", "&Info..."))
        self.act_logs.setText(_translate("win_revpicommander", "PLC &logs..."))
        self.act_options.setText(_translate("win_revpicommander", "PLC &options..."))
        self.act_program.setText(_translate("win_revpicommander", "PLC progra&m..."))
        self.act_developer.setText(_translate("win_revpicommander", "PLC de&veloper..."))
        self.act_pictory.setText(_translate("win_revpicommander", "piCtory configuraiton..."))
        self.act_disconnect.setText(_translate("win_revpicommander", "&Disconnect"))
        self.act_reset.setText(_translate("win_revpicommander", "Reset driver..."))
        self.act_simulator.setText(_translate("win_revpicommander", "RevPi si&mulator..."))
from . import ressources_rc


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    win_revpicommander = QtWidgets.QMainWindow()
    ui = Ui_win_revpicommander()
    ui.setupUi(win_revpicommander)
    win_revpicommander.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'aclmanager.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_diag_aclmanager(object):
    def setupUi(self, diag_aclmanager):
        diag_aclmanager.setObjectName("diag_aclmanager")
        diag_aclmanager.resize(454, 572)
        self.verticalLayout = QtWidgets.QVBoxLayout(diag_aclmanager)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gb_acls = QtWidgets.QGroupBox(diag_aclmanager)
        self.gb_acls.setObjectName("gb_acls")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.gb_acls)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tb_acls = QtWidgets.QTableWidget(self.gb_acls)
        self.tb_acls.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tb_acls.setTabKeyNavigation(False)
        self.tb_acls.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.tb_acls.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tb_acls.setWordWrap(False)
        self.tb_acls.setCornerButtonEnabled(False)
        self.tb_acls.setObjectName("tb_acls")
        self.tb_acls.setColumnCount(2)
        self.tb_acls.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tb_acls.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tb_acls.setHorizontalHeaderItem(1, item)
        self.tb_acls.horizontalHeader().setHighlightSections(False)
        self.tb_acls.horizontalHeader().setStretchLastSection(True)
        self.tb_acls.verticalHeader().setVisible(False)
        self.verticalLayout_2.addWidget(self.tb_acls)
        self.hl_acls = QtWidgets.QHBoxLayout()
        self.hl_acls.setObjectName("hl_acls")
        self.btn_edit = QtWidgets.QPushButton(self.gb_acls)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_edit.sizePolicy().hasHeightForWidth())
        self.btn_edit.setSizePolicy(sizePolicy)
        self.btn_edit.setObjectName("btn_edit")
        self.hl_acls.addWidget(self.btn_edit)
        self.btn_remove = QtWidgets.QPushButton(self.gb_acls)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_remove.sizePolicy().hasHeightForWidth())
        self.btn_remove.setSizePolicy(sizePolicy)
        self.btn_remove.setObjectName("btn_remove")
        self.hl_acls.addWidget(self.btn_remove)
        self.verticalLayout_2.addLayout(self.hl_acls)
        self.verticalLayout.addWidget(self.gb_acls)
        self.gb_edit = QtWidgets.QGroupBox(diag_aclmanager)
        self.gb_edit.setObjectName("gb_edit")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.gb_edit)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.btn_clear = QtWidgets.QPushButton(self.gb_edit)
        self.btn_clear.setObjectName("btn_clear")
        self.gridLayout_2.addWidget(self.btn_clear, 1, 0, 1, 1)
        self.btn_add = QtWidgets.QPushButton(self.gb_edit)
        self.btn_add.setObjectName("btn_add")
        self.gridLayout_2.addWidget(self.btn_add, 1, 1, 1, 1)
        self.fl_edit = QtWidgets.QFormLayout()
        self.fl_edit.setObjectName("fl_edit")
        self.lbl_ip = QtWidgets.QLabel(self.gb_edit)
        self.lbl_ip.setObjectName("lbl_ip")
        self.fl_edit.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.lbl_ip)
        self.lbl_level = QtWidgets.QLabel(self.gb_edit)
        self.lbl_level.setObjectName("lbl_level")
        self.fl_edit.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.lbl_level)
        self.cbb_level = QtWidgets.QComboBox(self.gb_edit)
        self.cbb_level.setObjectName("cbb_level")
        self.fl_edit.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.cbb_level)
        self.hl_ip = QtWidgets.QHBoxLayout()
        self.hl_ip.setObjectName("hl_ip")
        self.txt_ip_a = QtWidgets.QLineEdit(self.gb_edit)
        self.txt_ip_a.setMaxLength(3)
        self.txt_ip_a.setObjectName("txt_ip_a")
        self.hl_ip.addWidget(self.txt_ip_a)
        self.lbl_ip_a = QtWidgets.QLabel(self.gb_edit)
        self.lbl_ip_a.setText(".")
        self.lbl_ip_a.setObjectName("lbl_ip_a")
        self.hl_ip.addWidget(self.lbl_ip_a)
        self.txt_ip_b = QtWidgets.QLineEdit(self.gb_edit)
        self.txt_ip_b.setMaxLength(3)
        self.txt_ip_b.setObjectName("txt_ip_b")
        self.hl_ip.addWidget(self.txt_ip_b)
        self.lbl_ip_b = QtWidgets.QLabel(self.gb_edit)
        self.lbl_ip_b.setText(".")
        self.lbl_ip_b.setObjectName("lbl_ip_b")
        self.hl_ip.addWidget(self.lbl_ip_b)
        self.txt_ip_c = QtWidgets.QLineEdit(self.gb_edit)
        self.txt_ip_c.setMaxLength(3)
        self.txt_ip_c.setObjectName("txt_ip_c")
        self.hl_ip.addWidget(self.txt_ip_c)
        self.lbl_ip_c = QtWidgets.QLabel(self.gb_edit)
        self.lbl_ip_c.setText(".")
        self.lbl_ip_c.setObjectName("lbl_ip_c")
        self.hl_ip.addWidget(self.lbl_ip_c)
        self.txt_ip_d = QtWidgets.QLineEdit(self.gb_edit)
        self.txt_ip_d.setMaxLength(3)
        self.txt_ip_d.setObjectName("txt_ip_d")
        self.hl_ip.addWidget(self.txt_ip_d)
        self.fl_edit.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.hl_ip)
        self.gridLayout_2.addLayout(self.fl_edit, 0, 0, 1, 2)
        self.verticalLayout.addWidget(self.gb_edit)
        self.lbl_level_info = QtWidgets.QLabel(diag_aclmanager)
        self.lbl_level_info.setText("")
        self.lbl_level_info.setObjectName("lbl_level_info")
        self.verticalLayout.addWidget(self.lbl_level_info)
        self.btn_box = QtWidgets.QDialogButtonBox(diag_aclmanager)
        self.btn_box.setOrientation(QtCore.Qt.Horizontal)
        self.btn_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.btn_box.setObjectName("btn_box")
        self.verticalLayout.addWidget(self.btn_box)

        self.retranslateUi(diag_aclmanager)
        self.btn_box.accepted.connect(diag_aclmanager.accept) # type: ignore
        self.btn_box.rejected.connect(diag_aclmanager.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(diag_aclmanager)
        diag_aclmanager.setTabOrder(self.tb_acls, self.btn_edit)
        diag_aclmanager.setTabOrder(self.btn_edit, self.btn_remove)
        diag_aclmanager.setTabOrder(self.btn_remove, self.txt_ip_a)
        diag_aclmanager.setTabOrder(self.txt_ip_a, self.txt_ip_b)
        diag_aclmanager.setTabOrder(self.txt_ip_b, self.txt_ip_c)
        diag_aclmanager.setTabOrder(self.txt_ip_c, self.txt_ip_d)
        diag_aclmanager.setTabOrder(self.txt_ip_d, self.cbb_level)

    def retranslateUi(self, diag_aclmanager):
        _translate = QtCore.QCoreApplication.translate
        diag_aclmanager.setWindowTitle(_translate("diag_aclmanager", "IP access control list"))
        self.gb_acls.setTitle(_translate("diag_aclmanager", "Existing ACLs"))
        item = self.tb_acls.horizontalHeaderItem(0)
        item.setText(_translate("diag_aclmanager", "IP Address"))
        item = self.tb_acls.horizontalHeaderItem(1)
        item.setText(_translate("diag_aclmanager", "Access Level"))
        self.btn_edit.setText(_translate("diag_aclmanager", "&Edit"))
        self.btn_remove.setText(_translate("diag_aclmanager", "&Remove"))
        self.gb_edit.setTitle(_translate("diag_aclmanager", "Add / Edit access entry"))
        self.btn_clear.setText(_translate("diag_aclmanager", "Clear fields"))
        self.btn_add.setText(_translate("diag_aclmanager", "&Save entry"))
        self.lbl_ip.setText(_translate("diag_aclmanager", "IP address:"))
        self.lbl_level.setText(_translate("diag_aclmanager", "Access level:"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    diag_aclmanager = QtWidgets.QDialog()
    ui = Ui_diag_aclmanager()
    ui.setupUi(diag_aclmanager)
    diag_aclmanager.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'avahisearch.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_diag_search(object):
    def setupUi(self, diag_search):
        diag_search.setObjectName("diag_search")
        diag_search.resize(480, 360)
        self.gridLayout = QtWidgets.QGridLayout(diag_search)
        self.gridLayout.setObjectName("gridLayout")
        self.hl_header = QtWidgets.QHBoxLayout()
        self.hl_header.setObjectName("hl_header")
        self.lbl_search = QtWidgets.QLabel(diag_search)
        self.lbl_search.setObjectName("lbl_search")
        self.hl_header.addWidget(self.lbl_search)
        self.btn_restart = QtWidgets.QPushButton(diag_search)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_restart.sizePolicy().hasHeightForWidth())
        self.btn_restart.setSizePolicy(sizePolicy)
        self.btn_restart.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/action/ico/reload.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_restart.setIcon(icon)
        self.btn_restart.setIconSize(QtCore.QSize(24, 24))
        self.btn_restart.setObjectName("btn_restart")
        self.hl_header.addWidget(self.btn_restart)
        self.gridLayout.addLayout(self.hl_header, 0, 0, 1, 2)
        self.tb_revpi = QtWidgets.QTableWidget(diag_search)
        self.tb_revpi.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tb_revpi.setTabKeyNavigation(False)
        self.tb_revpi.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tb_revpi.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tb_revpi.setWordWrap(False)
        self.tb_revpi.setCornerButtonEnabled(False)
        self.tb_revpi.setObjectName("tb_revpi")
        self.tb_revpi.setColumnCount(2)
        self.tb_revpi.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tb_revpi.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tb_revpi.setHorizontalHeaderItem(1, item)
        self.tb_revpi.horizontalHeader().setHighlightSections(False)
        self.tb_revpi.horizontalHeader().setStretchLastSection(True)
        self.tb_revpi.verticalHeader().setVisible(False)
        self.gridLayout.addWidget(self.tb_revpi, 1, 0, 1, 2)
        self.btn_connect = QtWidgets.QPushButton(diag_search)
        self.btn_connect.setObjectName("btn_connect")
        self.gridLayout.addWidget(self.btn_connect, 2, 0, 1, 1)
        self.btn_save = QtWidgets.QPushButton(diag_search)
        self.btn_save.setObjectName("btn_save")
        self.gridLayout.addWidget(self.btn_save, 2, 1, 1, 1)
        self.btn_box = QtWidgets.QDialogButtonBox(diag_search)
        self.btn_box.setOrientation(QtCore.Qt.Horizontal)
        self.btn_box.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.btn_box.setObjectName("btn_box")
        self.gridLayout.addWidget(self.btn_box, 3, 0, 1, 2)
        self.act_copy_host = QtWidgets.QAction(diag_search)
        self.act_copy_host.setObjectName("act_copy_host")
        self.act_copy_ip = QtWidgets.QAction(diag_search)
        self.act_copy_ip.setObjectName("act_copy_ip")
        self.act_open_pictory = QtWidgets.QAction(diag_search)
        self.act_open_pictory.setObjectName("act_open_pictory")
        self.act_connect_ssh = QtWidgets.QAction(diag_search)
        self.act_connect_ssh.setObjectName("act_connect_ssh")
        self.act_connect_xmlrpc = QtWidgets.QAction(diag_search)
        self.act_connect_xmlrpc.setObjectName("act_connect_xmlrpc")
        self.act_connect = QtWidgets.QAction(diag_search)
        self.act_connect.setObjectName("act_connect")

        self.retranslateUi(diag_search)
        self.btn_box.rejected.connect(diag_search.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(diag_search)

    def retranslateUi(self, diag_search):
        _translate = QtCore.QCoreApplication.translate
        diag_search.setWindowTitle(_translate("diag_search", "Search Revolution Pi devices"))
        self.lbl_search.setText(_translate("diag_search", "Searching for Revolution Pi devices in your network..."))
        self.btn_restart.setToolTip(_translate("diag_search", "Restart search"))
        self.tb_revpi.setSortingEnabled(True)
        item = self.tb_revpi.horizontalHeaderItem(0)
        item.setText(_translate("diag_search", "Zero-conf name"))
        item = self.tb_revpi.horizontalHeaderItem(1)
        item.setText(_translate("diag_search", "IP address"))
        self.btn_connect.setText(_translate("diag_search", "&Connect to Revolution Pi"))
        self.btn_save.setText(_translate("diag_search", "&Save connection"))
        self.act_copy_host.setText(_translate("diag_search", "Copy host name"))
        self.act_copy_ip.setText(_translate("diag_search", "Copy IP address"))
        self.act_open_pictory.setText(_translate("diag_search", "Open piCtory"))
        self.act_connect_ssh.setText(_translate("diag_search", "Connect via SSH (recommended)"))
        self.act_connect_ssh.setToolTip(_translate("diag_search", "Establish a connection via encrypted SSH tunnel"))
        self.act_connect_xmlrpc.setText(_translate("diag_search", "Connect via XML-RPC"))
        self.act_connect_xmlrpc.setToolTip(_translate("diag_search", "You have to configure your Revolution Pi to accept this connections"))
        self.act_connect.setText(_translate("diag_search", "Connect"))
        self.act_connect.setToolTip(_translate("diag_search", "Connect to Revoluton Pi"))
from . import ressources_rc


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    diag_search = QtWidgets.QDialog()
    ui = Ui_diag_search()
    ui.setupUi(diag_search)
    diag_search.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""Options for MQTT system."""

__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2023 Sven Sager"
__license__ = "GPLv2"

from PyQt5 import QtGui, QtWidgets

from . import proginit as pi
from .ui.mqttmanager_ui import Ui_diag_mqtt


class MqttManager(QtWidgets.QDialog, Ui_diag_mqtt):
    """MQTT settings for option window."""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.setFixedSize(self.size())

        self.dc = {}

    def _changesdone(self):
        """
        Check for unsaved changes in dialog.

        :return: True, if unsaved changes was found
        """
        return (
            not self.dc or
            self.dc["mqttbasetopic"] != self.txt_basetopic.text() or
            self.dc["mqttsendinterval"] != self.sbx_sendinterval.value() or
            self.dc["mqttsend_on_event"] != int(self.cbx_send_on_event.isChecked()) or
            self.dc["mqttwrite_outputs"] != int(self.cbx_write_outputs.isChecked()) or
            self.dc["mqttbroker_address"] != self.txt_broker_address.text() or
            self.dc["mqtttls_set"] != int(self.cbx_tls_set.isChecked()) or
            self.dc["mqttport"] != self.sbx_port.value() or
            self.dc["mqttusername"] != self.txt_username.text() or
            self.dc["mqttpassword"] != self.txt_password.text() or
            self.dc["mqttclient_id"] != self.txt_client_id.text()
        )

    def _load_settings(self):
        """Load values to GUI widgets."""
        try:
            self.txt_basetopic.setText(self.dc["mqttbasetopic"])
            self.sbx_sendinterval.setValue(self.dc["mqttsendinterval"])
            self.dc["mqttsend_on_event"] = int(self.cbx_send_on_event.isChecked())
            self.dc["mqttwrite_outputs"] = int(self.cbx_write_outputs.isChecked())
            self.txt_broker_address.setText(self.dc["mqttbroker_address"])
            self.cbx_tls_set.setChecked(bool(self.dc["mqtttls_set"]))
            self.sbx_port.setValue(self.dc["mqttport"])
            self.txt_username.setText(self.dc["mqttusername"])
            self.txt_password.setText(self.dc["mqttpassword"])
            self.txt_client_id.setText(self.dc["mqttclient_id"])
        except Exception as e:
            pi.logger.exception(e)
            self.dc = {}
            return False
        return True

    def accept(self) -> None:
        """Save values to value dict."""
        self.dc["mqttbasetopic"] = self.txt_basetopic.text()
        self.dc["mqttsendinterval"] = self.sbx_sendinterval.value()
        self.dc["mqttsend_on_event"] = int(self.cbx_send_on_event.isChecked())
        self.dc["mqttwrite_outputs"] = int(self.cbx_write_outputs.isChecked())
        self.dc["mqttbroker_address"] = self.txt_broker_address.text()
        self.dc["mqtttls_set"] = int(self.cbx_tls_set.isChecked())
        self.dc["mqttport"] = self.sbx_port.value()
        self.dc["mqttusername"] = self.txt_username.text()
        self.dc["mqttpassword"] = self.txt_password.text()
        self.dc["mqttclient_id"] = self.txt_client_id.text()
        super().accept()

    def closeEvent(self, a0: QtGui.QCloseEvent) -> None:
        if self._changesdone():
            ask = QtWidgets.QMessageBox.question(
                self, self.tr("Question"), self.tr(
                    "Do you really want to quit? \nUnsaved changes will be lost."
                )
            ) == QtWidgets.QMessageBox.Yes
            if ask:
                self.reject()
            else:
                a0.ignore()

    def exec(self) -> int:
        """Load settings from dc and show dialog."""
        if not self._load_settings():
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Can not load the MQTT settings dialog. Missing values!"
                )
            )
            return QtWidgets.QDialog.Rejected
        return super().exec()

    def reject(self) -> None:
        """Reject settings."""
        self._load_settings()
        super().reject()

    @property
    def read_only(self):
        """Getter for read_only value."""
        return not self.txt_basetopic.isEnabled()

    @read_only.setter
    def read_only(self, value: bool):
        """Setter for read_only window."""
        self.txt_basetopic.setEnabled(not value)
        self.sbx_sendinterval.setEnabled(not value)
        self.cbx_send_on_event.setEnabled(not value)
        self.cbx_write_outputs.setEnabled(not value)
        self.txt_broker_address.setEnabled(not value)
        self.sbx_port.setEnabled(not value)
        self.cbx_tls_set.setEnabled(not value)
        self.txt_username.setEnabled(not value)
        self.txt_password.setEnabled(not value)
        self.txt_client_id.setEnabled(not value)
        if value:
            self.btn_box.setStandardButtons(
                QtWidgets.QDialogButtonBox.Cancel
            )
        else:
            self.btn_box.setStandardButtons(
                QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel
            )



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'revpiprogram.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_diag_program(object):
    def setupUi(self, diag_program):
        diag_program.setObjectName("diag_program")
        diag_program.resize(400, 501)
        self.verticalLayout = QtWidgets.QVBoxLayout(diag_program)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gb_plc = QtWidgets.QGroupBox(diag_program)
        self.gb_plc.setObjectName("gb_plc")
        self.gridLayout = QtWidgets.QGridLayout(self.gb_plc)
        self.gridLayout.setObjectName("gridLayout")
        self.cbb_plcprogram = QtWidgets.QComboBox(self.gb_plc)
        self.cbb_plcprogram.setObjectName("cbb_plcprogram")
        self.gridLayout.addWidget(self.cbb_plcprogram, 1, 0, 1, 3)
        self.rbn_pythonversion_3 = QtWidgets.QRadioButton(self.gb_plc)
        self.rbn_pythonversion_3.setObjectName("rbn_pythonversion_3")
        self.gridLayout.addWidget(self.rbn_pythonversion_3, 3, 2, 1, 1)
        self.cbx_plcworkdir_set_uid = QtWidgets.QCheckBox(self.gb_plc)
        self.cbx_plcworkdir_set_uid.setObjectName("cbx_plcworkdir_set_uid")
        self.gridLayout.addWidget(self.cbx_plcworkdir_set_uid, 4, 0, 1, 3)
        self.lbl_plcprogram = QtWidgets.QLabel(self.gb_plc)
        self.lbl_plcprogram.setObjectName("lbl_plcprogram")
        self.gridLayout.addWidget(self.lbl_plcprogram, 0, 0, 1, 3)
        self.lbl_pythonversion = QtWidgets.QLabel(self.gb_plc)
        self.lbl_pythonversion.setObjectName("lbl_pythonversion")
        self.gridLayout.addWidget(self.lbl_pythonversion, 3, 0, 1, 1)
        self.rbn_pythonversion_2 = QtWidgets.QRadioButton(self.gb_plc)
        self.rbn_pythonversion_2.setObjectName("rbn_pythonversion_2")
        self.gridLayout.addWidget(self.rbn_pythonversion_2, 3, 1, 1, 1)
        self.lbl_plcarguments = QtWidgets.QLabel(self.gb_plc)
        self.lbl_plcarguments.setObjectName("lbl_plcarguments")
        self.gridLayout.addWidget(self.lbl_plcarguments, 2, 0, 1, 1)
        self.txt_plcarguments = QtWidgets.QLineEdit(self.gb_plc)
        self.txt_plcarguments.setObjectName("txt_plcarguments")
        self.gridLayout.addWidget(self.txt_plcarguments, 2, 1, 1, 2)
        self.sbx_plcprogram_watchdog = QtWidgets.QSpinBox(self.gb_plc)
        self.sbx_plcprogram_watchdog.setMaximum(600)
        self.sbx_plcprogram_watchdog.setObjectName("sbx_plcprogram_watchdog")
        self.gridLayout.addWidget(self.sbx_plcprogram_watchdog, 5, 2, 1, 1)
        self.lbl_plcprogram_watchdog = QtWidgets.QLabel(self.gb_plc)
        self.lbl_plcprogram_watchdog.setObjectName("lbl_plcprogram_watchdog")
        self.gridLayout.addWidget(self.lbl_plcprogram_watchdog, 5, 0, 1, 2)
        self.verticalLayout.addWidget(self.gb_plc)
        self.cb_transfair = QtWidgets.QGroupBox(diag_program)
        self.cb_transfair.setObjectName("cb_transfair")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.cb_transfair)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.cbb_format = QtWidgets.QComboBox(self.cb_transfair)
        self.cbb_format.setObjectName("cbb_format")
        self.cbb_format.addItem("")
        self.cbb_format.addItem("")
        self.gridLayout_2.addWidget(self.cbb_format, 0, 1, 1, 1)
        self.btn_program_upload = QtWidgets.QPushButton(self.cb_transfair)
        self.btn_program_upload.setObjectName("btn_program_upload")
        self.gridLayout_2.addWidget(self.btn_program_upload, 3, 1, 1, 1)
        self.btn_program_download = QtWidgets.QPushButton(self.cb_transfair)
        self.btn_program_download.setObjectName("btn_program_download")
        self.gridLayout_2.addWidget(self.btn_program_download, 3, 0, 1, 1)
        self.lbl_format = QtWidgets.QLabel(self.cb_transfair)
        self.lbl_format.setObjectName("lbl_format")
        self.gridLayout_2.addWidget(self.lbl_format, 0, 0, 1, 1)
        self.cbx_pictory = QtWidgets.QCheckBox(self.cb_transfair)
        self.cbx_pictory.setObjectName("cbx_pictory")
        self.gridLayout_2.addWidget(self.cbx_pictory, 1, 0, 1, 2)
        self.cbx_clear = QtWidgets.QCheckBox(self.cb_transfair)
        self.cbx_clear.setObjectName("cbx_clear")
        self.gridLayout_2.addWidget(self.cbx_clear, 2, 0, 1, 2)
        self.verticalLayout.addWidget(self.cb_transfair)
        self.gb_control = QtWidgets.QGroupBox(diag_program)
        self.gb_control.setObjectName("gb_control")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.gb_control)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.btn_procimg_download = QtWidgets.QPushButton(self.gb_control)
        self.btn_procimg_download.setObjectName("btn_procimg_download")
        self.gridLayout_3.addWidget(self.btn_procimg_download, 1, 1, 1, 1)
        self.btn_pictory_download = QtWidgets.QPushButton(self.gb_control)
        self.btn_pictory_download.setObjectName("btn_pictory_download")
        self.gridLayout_3.addWidget(self.btn_pictory_download, 0, 1, 1, 1)
        self.btn_pictory_upload = QtWidgets.QPushButton(self.gb_control)
        self.btn_pictory_upload.setObjectName("btn_pictory_upload")
        self.gridLayout_3.addWidget(self.btn_pictory_upload, 0, 2, 1, 1)
        self.lbl_pictory = QtWidgets.QLabel(self.gb_control)
        self.lbl_pictory.setObjectName("lbl_pictory")
        self.gridLayout_3.addWidget(self.lbl_pictory, 0, 0, 1, 1)
        self.lbl_procimg = QtWidgets.QLabel(self.gb_control)
        self.lbl_procimg.setObjectName("lbl_procimg")
        self.gridLayout_3.addWidget(self.lbl_procimg, 1, 0, 1, 1)
        self.verticalLayout.addWidget(self.gb_control)
        self.btn_box = QtWidgets.QDialogButtonBox(diag_program)
        self.btn_box.setOrientation(QtCore.Qt.Horizontal)
        self.btn_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.btn_box.setObjectName("btn_box")
        self.verticalLayout.addWidget(self.btn_box)

        self.retranslateUi(diag_program)
        self.btn_box.accepted.connect(diag_program.accept) # type: ignore
        self.btn_box.rejected.connect(diag_program.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(diag_program)
        diag_program.setTabOrder(self.cbb_plcprogram, self.txt_plcarguments)
        diag_program.setTabOrder(self.txt_plcarguments, self.rbn_pythonversion_2)
        diag_program.setTabOrder(self.rbn_pythonversion_2, self.rbn_pythonversion_3)
        diag_program.setTabOrder(self.rbn_pythonversion_3, self.cbx_plcworkdir_set_uid)
        diag_program.setTabOrder(self.cbx_plcworkdir_set_uid, self.cbb_format)
        diag_program.setTabOrder(self.cbb_format, self.cbx_pictory)
        diag_program.setTabOrder(self.cbx_pictory, self.cbx_clear)
        diag_program.setTabOrder(self.cbx_clear, self.btn_program_download)
        diag_program.setTabOrder(self.btn_program_download, self.btn_program_upload)
        diag_program.setTabOrder(self.btn_program_upload, self.btn_pictory_download)
        diag_program.setTabOrder(self.btn_pictory_download, self.btn_pictory_upload)
        diag_program.setTabOrder(self.btn_pictory_upload, self.btn_procimg_download)

    def retranslateUi(self, diag_program):
        _translate = QtCore.QCoreApplication.translate
        diag_program.setWindowTitle(_translate("diag_program", "PLC program"))
        self.gb_plc.setTitle(_translate("diag_program", "PLC program"))
        self.rbn_pythonversion_3.setText(_translate("diag_program", "Python 3"))
        self.cbx_plcworkdir_set_uid.setText(_translate("diag_program", "Set write permissions for plc program to workdirectory"))
        self.lbl_plcprogram.setText(_translate("diag_program", "Python PLC start program:"))
        self.lbl_pythonversion.setText(_translate("diag_program", "Python version:"))
        self.rbn_pythonversion_2.setText(_translate("diag_program", "Python 2"))
        self.lbl_plcarguments.setText(_translate("diag_program", "Program arguments:"))
        self.sbx_plcprogram_watchdog.setSuffix(_translate("diag_program", " sec."))
        self.lbl_plcprogram_watchdog.setText(_translate("diag_program", "Software watchdog (0=disabled):"))
        self.cb_transfair.setTitle(_translate("diag_program", "Transfair PLC program"))
        self.cbb_format.setItemText(0, _translate("diag_program", "ZIP archive"))
        self.cbb_format.setItemText(1, _translate("diag_program", "TGZ archive"))
        self.btn_program_upload.setText(_translate("diag_program", "Upload"))
        self.btn_program_download.setText(_translate("diag_program", "Download"))
        self.lbl_format.setText(_translate("diag_program", "Transfair format:"))
        self.cbx_pictory.setText(_translate("diag_program", "Including piCtory configuration"))
        self.cbx_clear.setText(_translate("diag_program", "Remove all files on Revolution Pi before upload"))
        self.gb_control.setTitle(_translate("diag_program", "Control files"))
        self.btn_procimg_download.setText(_translate("diag_program", "Download"))
        self.btn_pictory_download.setText(_translate("diag_program", "Download"))
        self.btn_pictory_upload.setText(_translate("diag_program", "Upload"))
        self.lbl_pictory.setText(_translate("diag_program", "piCtory configuraiton"))
        self.lbl_procimg.setText(_translate("diag_program", "Process image from piControl0"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    diag_program = QtWidgets.QDialog()
    ui = Ui_diag_program()
    ui.setupUi(diag_program)
    diag_program.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""Global program initialization."""
__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2023 Sven Sager"
__license__ = "GPLv2"

import logging
import sys
from argparse import ArgumentParser
from configparser import ConfigParser
from os import R_OK, W_OK, access
from os.path import abspath, dirname, join

# Program name
programname = "revpicommander"

# Set to True, if you want to save config file
conf_rw = False

conf = ConfigParser()
logger = logging.getLogger()
pidfile = "/var/run/{0}.pid".format(programname)


def cleanup():
    """Clean up program."""
    # Shutdown logging system
    logging.shutdown()


def reconfigure_logger():
    """Configure logging module of program."""
    # Clear all log handler
    for lhandler in logger.handlers.copy():
        lhandler.close()
        logger.removeHandler(lhandler)

    # Create new log handler
    logformat = logging.Formatter(
        "{asctime} [{levelname:8}] {message}",
        datefmt="%Y-%m-%d %H:%M:%S", style="{"
    )
    lhandler = logging.StreamHandler(sys.stdout)
    lhandler.setFormatter(logformat)
    logger.addHandler(lhandler)

    if "logfile" in pargs and pargs.logfile is not None:
        # Write logs to a logfile
        lhandler = logging.FileHandler(filename=pargs.logfile)
        lhandler.setFormatter(logformat)
        logger.addHandler(lhandler)

    # Loglevel auswerten
    if pargs.verbose == 1:
        loglevel = logging.INFO
    elif pargs.verbose > 1:
        loglevel = logging.DEBUG
    else:
        loglevel = logging.WARNING
    logger.setLevel(loglevel)


def reload_conf():
    """Reload config file."""
    if "conffile" in pargs:

        # Check config file
        if not access(pargs.conffile, R_OK):
            raise RuntimeError(
                "can not access config file '{0}'".format(pargs.conffile)
            )
        if conf_rw and not access(pargs.conffile, W_OK):
            raise RuntimeError(
                "can not write to config file '{0}'".format(pargs.conffile)
            )

        # Create global config
        global conf
        logger.info("loading config file: {0}".format(pargs.conffile))
        conf.read(pargs.conffile)


# Generate command arguments of the program
parser = ArgumentParser(
    prog=programname,
    description="Program description"
)
parser.add_argument(
    "-f", "--logfile", dest="logfile",
    help="Save log entries to this file"
)
parser.add_argument(
    "-v", "--verbose", action="count", dest="verbose", default=0,
    help="Switch on verbose logging"
)
# The __main__ script will process the version number argument
parser.add_argument("--version", action="store_true", help="Print version number of program and exit")
pargs = parser.parse_args()

# Check important objects and set to default if they do not exists
if "verbose" not in pargs:
    pargs.verbose = 0

# Get absolute paths
pwd = abspath(".")

# Configure logger
if "logfile" in pargs and pargs.logfile is not None \
        and dirname(pargs.logfile) == "":
    pargs.logfile = join(pwd, pargs.logfile)
reconfigure_logger()

# Initialize configparser of globalconfig
if "conffile" in pargs and dirname(pargs.conffile) == "":
    pargs.conffile = join(pwd, pargs.conffile)

# Load configuration - Comment out, if you do that in your own program
reload_conf()



# -*- coding: utf-8 -*-
"""Revolution Pi search with zeroconf."""
__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2023 Sven Sager"
__license__ = "GPLv2"

import webbrowser
from re import compile
from sys import platform

from PyQt5 import QtCore, QtGui, QtWidgets
from zeroconf import IPVersion, ServiceBrowser, Zeroconf

from . import helper
from . import proginit as pi
from .helper import RevPiSettings, WidgetData, all_revpi_settings
from .ui.avahisearch_ui import Ui_diag_search


class AvahiSearchThread(QtCore.QThread):
    """Search thread for Revolution Pi with installed RevPiPyLoad."""
    added = QtCore.pyqtSignal(str, str, int, str, str)
    removed = QtCore.pyqtSignal(str, str)
    updated = QtCore.pyqtSignal(str, str, int, str, str)

    def __init__(self, parent=None):
        super().__init__(parent)
        self._cycle_wait_ms = 1000

        self.re_posix = compile(
            r"(?P<ip>(\d{1,3}\.){3}\d{1,3}).*"
            r"(?P<mac>([0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2})"
        )

    def remove_service(self, zeroconf: Zeroconf, conf_type: str, name: str) -> None:
        """Revolution Pi disappeared."""
        pi.logger.debug("AvahiSearchThread.remove_service")
        self.removed.emit(name, conf_type)

    def add_service(self, zeroconf: Zeroconf, conf_type: str, name: str) -> None:
        """New Revolution Pi found."""
        pi.logger.debug("AvahiSearchThread.add_service")
        info = zeroconf.get_service_info(conf_type, name)
        if not info:
            return

        for ip in info.parsed_addresses(IPVersion.V4Only):
            self.added.emit(name, info.server, info.port, conf_type, ip)

    def update_service(self, zeroconf: Zeroconf, conf_type: str, name: str) -> None:
        """New data of revolution pi"""
        pi.logger.debug("AvahiSearchThread.add_service")
        info = zeroconf.get_service_info(conf_type, name)
        if not info:
            return

        for ip in info.parsed_addresses(IPVersion.V4Only):
            self.updated.emit(name, info.server, info.port, conf_type, ip)

    def run(self) -> None:
        pi.logger.debug("Started zero conf discovery.")
        zeroconf = Zeroconf()
        revpi_browser = ServiceBrowser(zeroconf, "_revpipyload._tcp.local.", self)
        while not self.isInterruptionRequested():
            # Just hanging around :)
            self.msleep(self._cycle_wait_ms)
        zeroconf.close()
        pi.logger.debug("Stopped zero conf discovery.")


class AvahiSearch(QtWidgets.QDialog, Ui_diag_search):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)

        # Global variables to let parent decide other actions
        self.connect_settings = None
        self.just_save = False

        # Local variables
        self.clipboard = QtGui.QGuiApplication.clipboard()
        self._th_zero_conf = AvahiSearchThread(self)

        self.tb_revpi.setColumnWidth(0, 250)
        self.btn_connect.setEnabled(False)
        self.btn_save.setEnabled(False)

        self.restoreGeometry(helper.settings.value("avahisearch/geo", b''))
        column_sizes = helper.settings.value("avahisearch/column_sizes", [], type=list)
        if len(column_sizes) == self.tb_revpi.columnCount():
            for i in range(self.tb_revpi.columnCount()):
                self.tb_revpi.setColumnWidth(i, int(column_sizes[i]))

        # Global context menus
        self.cm_connect_actions = QtWidgets.QMenu(self)
        self.cm_connect_actions.addAction(self.act_connect_ssh)
        self.cm_connect_actions.addAction(self.act_connect_xmlrpc)

        self.cm_quick_actions = QtWidgets.QMenu(self)
        self.cm_quick_actions.addAction(self.act_connect)
        self.cm_quick_actions.addAction(self.act_connect_ssh)
        self.cm_quick_actions.addAction(self.act_connect_xmlrpc)
        self.cm_quick_actions.addSeparator()
        self.cm_quick_actions.addAction(self.act_open_pictory)
        self.cm_quick_actions.addSeparator()
        self.cm_quick_actions.addAction(self.act_copy_host)
        self.cm_quick_actions.addAction(self.act_copy_ip)

        self.tb_revpi.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tb_revpi.customContextMenuRequested.connect(self._cm_quick_exec)

    @QtCore.pyqtSlot(QtCore.QPoint)
    def _cm_quick_exec(self, position: QtCore.QPoint) -> None:
        selected_items = self.tb_revpi.selectedItems()
        if not selected_items:
            return
        item = selected_items[0]

        revpi_settings = bool(item.data(WidgetData.revpi_settings))
        self.act_connect.setVisible(revpi_settings)
        self.act_connect_ssh.setVisible(not revpi_settings)
        self.act_connect_xmlrpc.setVisible(not revpi_settings)

        sender = self.sender()
        self.cm_quick_actions.exec(sender.mapToGlobal(position))

        self.act_connect.setVisible(True)
        self.act_connect_ssh.setVisible(True)
        self.act_connect_xmlrpc.setVisible(True)

    @staticmethod
    def _find_settings(address: str):
        """Find all settings with known avahi_id."""
        return [
            revpi_setting
            for revpi_setting in all_revpi_settings()
            if revpi_setting.address.lower() == address.lower()
        ]

    def _restart_search(self) -> None:
        """Clean up and restart search thread."""
        while self.tb_revpi.rowCount() > 0:
            # Remove each row, a .clean would destroy the columns
            self.tb_revpi.removeRow(0)

        self._th_zero_conf.requestInterruption()

        self._th_zero_conf = AvahiSearchThread(self)
        self._th_zero_conf.added.connect(self.on_avahi_added)
        self._th_zero_conf.updated.connect(self.on_avahi_added)
        self._th_zero_conf.removed.connect(self.on_avahi_removed)
        self._th_zero_conf.start()

    def _create_settings_object(self, row: int, ssh_tunnel: bool) -> RevPiSettings or None:
        """
        Create settings object from given row to settings.

        :param row: Row with connection data
        :param ssh_tunnel: Save as SSH tunnel connection
        :return: RevPi settings with data from avahi search and default values
        """
        item = self.tb_revpi.item(row, 0)
        if not item:
            return None

        settings = RevPiSettings()
        settings.folder = self.tr("Auto discovered")
        settings.name = item.data(WidgetData.host_name)
        settings.address = item.data(WidgetData.address)
        settings.port = item.data(WidgetData.port)
        settings.ssh_use_tunnel = ssh_tunnel

        return settings

    def closeEvent(self, a0: QtGui.QCloseEvent) -> None:
        helper.settings.setValue("avahisearch/geo", self.saveGeometry())
        helper.settings.setValue("avahisearch/column_sizes", [
            self.tb_revpi.columnWidth(i)
            for i in range(self.tb_revpi.columnCount())
        ])

    def exec(self) -> int:
        self.connect_settings = None
        self.just_save = False
        self._restart_search()
        rc = super().exec()
        self._th_zero_conf.requestInterruption()
        return rc

    @QtCore.pyqtSlot()
    def on_act_connect_triggered(self) -> None:
        """Connect via existing settings or ask for type."""
        pi.logger.debug("AvahiSearch.on_act_connect_triggered")
        selected_items = self.tb_revpi.selectedItems()
        if not selected_items:
            return
        item = selected_items[0]

        revpi_settings = item.data(WidgetData.revpi_settings)  # type: RevPiSettings
        if not revpi_settings:
            return
        self.connect_settings = revpi_settings
        self.accept()

    @QtCore.pyqtSlot()
    def on_act_connect_ssh_triggered(self) -> None:
        """Create new revpi settings with ssh, save and connect."""
        pi.logger.debug("AvahiSearch.on_act_connect_ssh_triggered")
        if self.tb_revpi.currentRow() == -1:
            return

        revpi_settings = self._create_settings_object(self.tb_revpi.currentRow(), True)
        revpi_settings.save_settings()
        self.connect_settings = revpi_settings
        self.accept()

    @QtCore.pyqtSlot()
    def on_act_connect_xmlrpc_triggered(self) -> None:
        """Create new revpi settings with XML-RPC, save and connect."""
        pi.logger.debug("AvahiSearch.on_act_connect_xmlrpc_triggered")
        if self.tb_revpi.currentRow() == -1:
            return

        revpi_settings = self._create_settings_object(self.tb_revpi.currentRow(), False)
        revpi_settings.save_settings()
        self.connect_settings = revpi_settings
        self.accept()

    @QtCore.pyqtSlot()
    def on_act_copy_host_triggered(self) -> None:
        """Copy hostname of selected item to clipboard."""
        selected_items = self.tb_revpi.selectedItems()
        if not selected_items:
            return
        item = selected_items[0]

        # Use just the hostname on Windows systems, it can not resolve .local addresses
        self.clipboard.setText(
            item.data(WidgetData.host_name) if platform == "win32"
            else item.data(WidgetData.host_name_full)
        )

    @QtCore.pyqtSlot()
    def on_act_copy_ip_triggered(self) -> None:
        """Copy ip address of selected item to clipboard."""
        selected_items = self.tb_revpi.selectedItems()
        if not selected_items:
            return
        item = selected_items[0]
        self.clipboard.setText(item.data(WidgetData.address))

    @QtCore.pyqtSlot()
    def on_act_open_pictory_triggered(self) -> None:
        """Open piCtory in default browser of operating system."""
        selected_items = self.tb_revpi.selectedItems()
        if not selected_items:
            return
        item = selected_items[0]

        # We should use the hostname on macOS to let safari connect in link local mode (for linux nice too)
        webbrowser.open("http://{0}/".format(item.data(
            WidgetData.address if platform == "win32"
            else WidgetData.host_name_full
        )))

    @QtCore.pyqtSlot(str, str, int, str, str)
    def on_avahi_added(self, avahi_id: str, server: str, port: int, conf_type: str, ip: str) -> None:
        """New Revolution Pi found."""

        def update_tb_revpi_row(row_index: int):
            host_name_full = server[:-1]
            host_name = host_name_full[:host_name_full.find(".")]

            item_name = self.tb_revpi.item(row_index, 0)
            item_name.setData(WidgetData.address, ip)
            item_name.setData(WidgetData.port, port)
            item_name.setData(WidgetData.host_name_full, host_name_full)
            item_name.setData(WidgetData.host_name, host_name)

            revpi_settings = item_name.data(WidgetData.revpi_settings)  # type: RevPiSettings
            if revpi_settings:
                # Generate the name of saved revpi and show the avahi-name in brackets
                settings_text = "{0}/{1}".format(revpi_settings.folder, revpi_settings.name) \
                    if revpi_settings.folder \
                    else revpi_settings.name
                if revpi_settings.ssh_use_tunnel:
                    settings_text += self.tr(" over SSH")
                item_name.setText("{0} ({1})".format(settings_text, host_name))
            else:
                item_name.setText(host_name)
            item_name.setToolTip(item_name.text())

            item_ip = self.tb_revpi.item(row_index, 1)
            item_ip.setText(ip)
            item_ip.setToolTip(item_name.text())

        lst_existing = self._find_settings(ip)

        exists = False
        for i in range(self.tb_revpi.rowCount()):
            item_tb_revpi = self.tb_revpi.item(i, 0)
            if item_tb_revpi.data(WidgetData.object_name) == avahi_id:
                # Object already discovered
                update_tb_revpi_row(i)
                exists = True

        if not exists:
            for known_settings in lst_existing or [None]:
                item_name = QtWidgets.QTableWidgetItem()

                item_name.setIcon(QtGui.QIcon(":/main/ico/cpu.ico"))
                item_name.setData(WidgetData.object_name, avahi_id)
                item_name.setData(WidgetData.revpi_settings, known_settings)

                index = self.tb_revpi.rowCount()
                self.tb_revpi.insertRow(index)
                self.tb_revpi.setItem(index, 0, item_name)
                self.tb_revpi.setItem(index, 1, QtWidgets.QTableWidgetItem())

                update_tb_revpi_row(index)

    @QtCore.pyqtSlot(str, str)
    def on_avahi_removed(self, avahi_id: str, conf_type: str) -> None:
        """Revolution Pi disappeared."""
        for i in range(self.tb_revpi.rowCount()):
            if self.tb_revpi.item(i, 0).data(WidgetData.object_name) == avahi_id:
                self.tb_revpi.removeRow(i)
                break

    @QtCore.pyqtSlot(int, int)
    def on_tb_revpi_cellDoubleClicked(self, row: int, column: int) -> None:
        """Connect to double-clicked Revolution Pi."""
        pi.logger.debug("AvahiSearch.on_tb_revpi_cellDoubleClicked")
        selected_items = self.tb_revpi.selectedItems()
        if not selected_items:
            return
        item = selected_items[0]

        revpi_settings = bool(item.data(WidgetData.revpi_settings))
        if revpi_settings:
            self.act_connect.trigger()
        else:
            cur = QtGui.QCursor()
            self.cm_connect_actions.exec(cur.pos())

    @QtCore.pyqtSlot(int, int, int, int)
    def on_tb_revpi_currentCellChanged(self, row: int, column: int, last_row: int, last_column: int) -> None:
        """Manage state of buttons."""
        self.btn_connect.setEnabled(row >= 0)
        self.btn_save.setEnabled(row >= 0)

    @QtCore.pyqtSlot()
    def on_btn_connect_clicked(self) -> None:
        """Connect to selected Revolution Pi."""
        pi.logger.debug("AvahiSearch.on_btn_connect_clicked")
        selected_items = self.tb_revpi.selectedItems()
        if not selected_items:
            return
        item = selected_items[0]

        revpi_settings = bool(item.data(WidgetData.revpi_settings))
        if revpi_settings:
            self.act_connect.trigger()
        else:
            pos_context_menu = self.btn_connect.pos()
            pos_context_menu.setY(pos_context_menu.y() + self.btn_connect.height())
            self.cm_connect_actions.exec(self.mapToGlobal(pos_context_menu))

    @QtCore.pyqtSlot()
    def on_btn_save_clicked(self) -> None:
        """Save selected Revolution Pi."""
        pi.logger.debug("AvahiSearch.on_btn_save_clicked")
        row_index = self.tb_revpi.currentRow()
        if row_index == -1:
            return
        self.connect_settings = self._create_settings_object(row_index, True)
        self.just_save = True
        self.accept()

    @QtCore.pyqtSlot()
    def on_btn_restart_clicked(self) -> None:
        """Clean up and restart search thread."""
        self._restart_search()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'revpilogfile.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_win_revpilogfile(object):
    def setupUi(self, win_revpilogfile):
        win_revpilogfile.setObjectName("win_revpilogfile")
        win_revpilogfile.resize(796, 347)
        self.centralwidget = QtWidgets.QWidget(win_revpilogfile)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.cbx_stay_on_top = QtWidgets.QCheckBox(self.centralwidget)
        self.cbx_stay_on_top.setObjectName("cbx_stay_on_top")
        self.gridLayout_3.addWidget(self.cbx_stay_on_top, 1, 0, 1, 1)
        self.cbx_wrap = QtWidgets.QCheckBox(self.centralwidget)
        self.cbx_wrap.setObjectName("cbx_wrap")
        self.gridLayout_3.addWidget(self.cbx_wrap, 1, 1, 1, 1)
        self.splitter = QtWidgets.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.gridLayoutWidget = QtWidgets.QWidget(self.splitter)
        self.gridLayoutWidget.setObjectName("gridLayoutWidget")
        self.grid_daemon = QtWidgets.QGridLayout(self.gridLayoutWidget)
        self.grid_daemon.setContentsMargins(0, 0, 0, 0)
        self.grid_daemon.setObjectName("grid_daemon")
        self.lbl_daemon = QtWidgets.QLabel(self.gridLayoutWidget)
        self.lbl_daemon.setObjectName("lbl_daemon")
        self.grid_daemon.addWidget(self.lbl_daemon, 0, 0, 1, 1)
        self.btn_daemon = QtWidgets.QPushButton(self.gridLayoutWidget)
        self.btn_daemon.setObjectName("btn_daemon")
        self.grid_daemon.addWidget(self.btn_daemon, 0, 1, 1, 1)
        self.txt_daemon = QtWidgets.QPlainTextEdit(self.gridLayoutWidget)
        self.txt_daemon.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.txt_daemon.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.txt_daemon.setTabChangesFocus(True)
        self.txt_daemon.setUndoRedoEnabled(False)
        self.txt_daemon.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.txt_daemon.setReadOnly(True)
        self.txt_daemon.setObjectName("txt_daemon")
        self.grid_daemon.addWidget(self.txt_daemon, 1, 0, 1, 2)
        self.grid_daemon.setColumnStretch(0, 1)
        self.gridLayoutWidget_2 = QtWidgets.QWidget(self.splitter)
        self.gridLayoutWidget_2.setObjectName("gridLayoutWidget_2")
        self.grid_app = QtWidgets.QGridLayout(self.gridLayoutWidget_2)
        self.grid_app.setContentsMargins(0, 0, 0, 0)
        self.grid_app.setObjectName("grid_app")
        self.btn_app = QtWidgets.QPushButton(self.gridLayoutWidget_2)
        self.btn_app.setObjectName("btn_app")
        self.grid_app.addWidget(self.btn_app, 0, 1, 1, 1)
        self.lbl_app = QtWidgets.QLabel(self.gridLayoutWidget_2)
        self.lbl_app.setObjectName("lbl_app")
        self.grid_app.addWidget(self.lbl_app, 0, 0, 1, 1)
        self.txt_app = QtWidgets.QPlainTextEdit(self.gridLayoutWidget_2)
        self.txt_app.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.txt_app.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.txt_app.setTabChangesFocus(True)
        self.txt_app.setUndoRedoEnabled(False)
        self.txt_app.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.txt_app.setReadOnly(True)
        self.txt_app.setObjectName("txt_app")
        self.grid_app.addWidget(self.txt_app, 1, 0, 1, 2)
        self.grid_app.setColumnStretch(0, 1)
        self.gridLayout_3.addWidget(self.splitter, 0, 0, 1, 2)
        win_revpilogfile.setCentralWidget(self.centralwidget)

        self.retranslateUi(win_revpilogfile)
        QtCore.QMetaObject.connectSlotsByName(win_revpilogfile)

    def retranslateUi(self, win_revpilogfile):
        _translate = QtCore.QCoreApplication.translate
        win_revpilogfile.setWindowTitle(_translate("win_revpilogfile", "RevPi Python PLC Logfiles"))
        self.cbx_stay_on_top.setText(_translate("win_revpilogfile", "Stay on top of all windows"))
        self.cbx_wrap.setText(_translate("win_revpilogfile", "Linewrap"))
        self.lbl_daemon.setText(_translate("win_revpilogfile", "RevPiPyLoad - Logfile"))
        self.btn_daemon.setText(_translate("win_revpilogfile", "Clear view"))
        self.btn_app.setText(_translate("win_revpilogfile", "Clear view"))
        self.lbl_app.setText(_translate("win_revpilogfile", "Python PLC program - Logfile"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    win_revpilogfile = QtWidgets.QMainWindow()
    ui = Ui_win_revpilogfile()
    ui.setupUi(win_revpilogfile)
    win_revpilogfile.show()
    sys.exit(app.exec_())



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""RevPiCommander main program."""

__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2018 Sven Sager"
__license__ = "GPLv2"

import webbrowser
from os.path import dirname, join

from PyQt5 import QtCore, QtGui, QtWidgets

from . import __version__
from . import helper
from . import proginit as pi
from . import revpilogfile
from .avahisearch import AvahiSearch
from .backgroundworker import BackgroundWaiter
from .debugcontrol import DebugControl
from .helper import ConnectionFail, RevPiSettings
from .revpifiles import RevPiFiles
from .revpiinfo import RevPiInfo
from .revpioption import RevPiOption
from .revpiplclist import RevPiPlcList
from .revpiprogram import RevPiProgram
from .simulator import Simulator
from .sshauth import SSHAuth
from .ui.revpicommander_ui import Ui_win_revpicommander


class ConnectingPyload(QtCore.QThread):
    """
    Try to establish a connection in background.

    The pyload_connect function will emit signals for error or successful connect. This
    signals will be used to show error messages and return to this function, if the
    authentication failed.
    """

    def __init__(self, revpi_settings: RevPiSettings, ssh_password="", parent=None):
        super().__init__(parent)
        self._revpi_settings = revpi_settings
        self._ssh_password = ssh_password

    def run(self) -> None:
        helper.cm.pyload_connect(self._revpi_settings, self._ssh_password)


class RevPiCommander(QtWidgets.QMainWindow, Ui_win_revpicommander):
    """Main application of RevPiCommander."""

    def __init__(self, parent=None):
        """Init main program."""
        super().__init__(parent)
        self.setupUi(self)

        self.wid_debugcontrol = None  # type: DebugControl
        """Holds the widget of DebugControl."""
        self.simulating = False
        """True, if simulation is running."""
        self.dict_men_connections_subfolder = {}
        """Submenus for folder entries."""

        self._set_gui_control_states()
        self._load_men_connections()

        # Load sub windows
        self.diag_connections = RevPiPlcList(self)
        self.diag_search = AvahiSearch(self)
        self.diag_info = RevPiInfo(__version__, self)
        self.diag_options = RevPiOption(self)
        self.diag_program = RevPiProgram(self)
        self.win_files = RevPiFiles(self)
        self.win_log = revpilogfile.RevPiLogfile(self)

        self.btn_plc_logs.clicked.connect(self.on_act_logs_triggered)

        helper.cm.connect_error.connect(self.on_cm_connect_error)
        helper.cm.connection_disconnected.connect(self.on_cm_connection_disconnected)
        helper.cm.connection_disconnecting.connect(self.on_cm_connection_disconnecting)
        helper.cm.connection_established.connect(self.on_cm_connection_established)
        helper.cm.connection_error_observed.connect(self.on_cm_connection_error_observed)
        helper.cm.status_changed.connect(self.on_cm_status_changed)

        self.restoreGeometry(helper.settings.value("revpicommander/geo", b''))

        self.setWindowFlag(QtCore.Qt.WindowMaximizeButtonHint, False)

    def closeEvent(self, a0: QtGui.QCloseEvent) -> None:
        pi.logger.debug("RevPiCommander.closeEvent")
        helper.cm.pyload_disconnect()
        helper.settings.setValue("revpicommander/geo", self.saveGeometry())

    def _set_gui_control_states(self):
        """Setup states of actions and buttons."""
        if helper.cm.simulating:
            self.btn_plc_stop.setEnabled(True)  # Stop simulator
            self.btn_plc_restart.setEnabled(True)  # Reset simulator values
            self.btn_plc_debug.setEnabled(True)

        else:
            connected = helper.cm.connected
            self.men_plc.setEnabled(connected)
            self.act_logs.setEnabled(connected)
            self.act_options.setEnabled(connected and helper.cm.xml_mode >= 2)
            self.act_program.setEnabled(connected and helper.cm.xml_mode >= 2)
            self.act_developer.setEnabled(connected and helper.cm.xml_mode >= 3)
            self.act_pictory.setEnabled(connected)
            self.act_reset.setEnabled(connected and helper.cm.xml_mode >= 3)
            self.act_disconnect.setEnabled(connected)
            self.btn_plc_start.setEnabled(connected)
            self.btn_plc_stop.setEnabled(connected)
            self.btn_plc_restart.setEnabled(connected)
            self.btn_plc_logs.setEnabled(connected)
            self.btn_plc_debug.setEnabled(connected and helper.cm.xml_mode >= 1)

    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    # region #      REGION: Connection management

    @QtCore.pyqtSlot(str, str, ConnectionFail, RevPiSettings)
    def on_cm_connect_error(self, title: str, text: str, fail_code: ConnectionFail, revpi_settings: RevPiSettings):
        """
        Slot to get information of pyload_connect connection errors.

        :param title: Title of error message
        :param text: Text of error message
        :param fail_code: Type of error
        :param revpi_settings: Settings of the revpi with the error
        """
        if fail_code is ConnectionFail.NO_XML_RPC_VIA_TUNNEL:
            # If RevPiPyLoad is not running, we can try to activate it via ssh
            QtWidgets.QMessageBox.information(
                self, self.tr("Information"), self.tr(
                    "Can not connect to RevPiPyLoad service through SSH tunnel!\n\n"
                    "We are trying to activate this service now and reconnect. The settings can be "
                    "changed at any time via 'webstatus'."
                ),
            )
            revpi_settings.ssh_enable_revpipyload = True
            self._pyload_connect(revpi_settings, False)
        else:
            QtWidgets.QMessageBox.critical(self, title, text)
            if fail_code is ConnectionFail.SSH_AUTH:
                # On failed credentials, we try to connect again and remove password form keychain, if exists
                self._pyload_connect(revpi_settings, True)

    @QtCore.pyqtSlot(str)
    def on_cm_connection_error_observed(self, message: str):
        """
        Connection error occurred in connection manager.

        :param message: Error message
        """
        self.statusbar.showMessage(message, 15000)

    @QtCore.pyqtSlot()
    def on_cm_connection_disconnected(self):
        """Connection of connection manager was disconnected."""
        pi.logger.debug("RevPiCommander.on_cm_connection_disconnected")

        self._set_gui_control_states()
        self.txt_host.setVisible(True)
        self.txt_host.clear()
        self.txt_connection.clear()

    @QtCore.pyqtSlot()
    def on_cm_connection_disconnecting(self):
        """Connection of connection manager will now disconnect."""
        pi.logger.debug("RevPiCommander.on_cm_connection_disconnecting")

        # This will remove the widgets in the button functions
        self.btn_plc_debug.setChecked(False)

        self.diag_info.reject()
        self.diag_options.reject()
        self.diag_program.reject()
        self.win_files.close()
        self.win_files.deleteLater()

        self.centralwidget.adjustSize()
        self.adjustSize()

    @QtCore.pyqtSlot()
    def on_cm_connection_established(self):
        """Connection manager established a new connection and loaded values."""
        pi.logger.debug("RevPiCommander.on_cm_connection_established")

        self._set_gui_control_states()
        if helper.cm.simulating:
            self.txt_host.setVisible(False)
            self.txt_connection.setText("configrsc=\"{0}\", procimg=\"{1}\"".format(
                helper.cm.simulating_configrsc,
                helper.cm.simulating_procimg,
            ))
        else:
            self.txt_host.setText(helper.cm.settings.name)
            self.txt_connection.setText(helper.cm.settings.address)
        self.win_files = RevPiFiles(self)

    @QtCore.pyqtSlot(str, str)
    def on_cm_status_changed(self, text: str, color: str):
        """PLC program status from Revolution Pi."""
        self.txt_status.setText(text)
        self.txt_status.setStyleSheet("background: {0}".format(color))

    # endregion # # # # #

    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    # region #      REGION: Menu entries

    def _load_men_connections(self):
        """Build up connections menu."""

        self.men_connections.clear()
        self.dict_men_connections_subfolder.clear()

        for settings in helper.all_revpi_settings():  # type: RevPiSettings
            if settings.folder:
                if settings.folder not in self.dict_men_connections_subfolder:
                    men_sub = QtWidgets.QMenu(self.men_connections)
                    men_sub.setTitle(settings.folder)
                    self.dict_men_connections_subfolder[settings.folder] = men_sub
                    self.men_connections.addMenu(men_sub)
                parent_menu = self.dict_men_connections_subfolder[settings.folder]
            else:
                parent_menu = self.men_connections

            display_name = settings.name
            if settings.name != settings.address:
                display_name += " [{0}]".format(settings.address)
            if settings.ssh_use_tunnel:
                display_name += " (SSH)"

            act = QtWidgets.QAction(parent_menu)
            act.setText(display_name)
            act.setData(settings)
            act.setToolTip("{0}:{1}".format(settings.address, settings.port))
            parent_menu.addAction(act)

    def _pyload_connect(self, revpi_settings: RevPiSettings, remove_saved_ssh_password=False):
        """
        Try to async establish a connection to Revolution Pi.

        :param revpi_settings: RevPi settings object
        :param remove_saved_ssh_password: Remove password from keychain
        """
        ssh_password = ""

        diag_connecting = BackgroundWaiter(
            revpi_settings.timeout,
            self.tr("Establish a connection to the Revolution Pi..."),
            self,
            self.tr("Revolution Pi connected!"),
        )
        helper.cm.connection_established.connect(diag_connecting.requestInterruption)

        if revpi_settings.ssh_use_tunnel:
            diag_ssh_auth = SSHAuth(
                revpi_settings.ssh_user,
                "{0}.{1}_{2}".format(
                    helper.settings.applicationName(),
                    helper.settings.organizationName(),
                    revpi_settings.internal_id),
                self,
            )

            if remove_saved_ssh_password and revpi_settings.ssh_saved_password:
                diag_ssh_auth.remove_saved_password()
                revpi_settings.ssh_saved_password = False

            # Doesn't matter what user selects, we have to save settings to sync keychain and values
            if diag_ssh_auth.exec() != QtWidgets.QDialog.Accepted:
                revpi_settings.save_settings()
                return

            revpi_settings.ssh_saved_password = diag_ssh_auth.in_keyring
            revpi_settings.ssh_user = diag_ssh_auth.username
            ssh_password = diag_ssh_auth.password
            revpi_settings.save_settings()

        # Connect in background and show the connecting dialog to user
        th_connecting = ConnectingPyload(revpi_settings, ssh_password, self)
        th_connecting.finished.connect(diag_connecting.requestInterruption)
        th_connecting.start()

        diag_connecting.exec_dialog(self.tr("Connecting..."), False)

    @QtCore.pyqtSlot()
    def on_act_connections_triggered(self):
        """Edit saved connections to Revolution Pi devices."""
        if self.diag_connections.exec() == QtWidgets.QDialog.Accepted:
            self._load_men_connections()

    @QtCore.pyqtSlot()
    def on_act_search_triggered(self):
        """Search for Revolution Pi with zero conf."""
        if self.diag_search.exec() == QtWidgets.QDialog.Accepted:
            if self.diag_search.connect_settings:
                if self.diag_search.just_save:
                    self.diag_connections.exec_with_presets(self.diag_search.connect_settings)
                else:
                    self._pyload_connect(self.diag_search.connect_settings)

        self._load_men_connections()

    @QtCore.pyqtSlot()
    def on_act_simulator_triggered(self):
        """Start the simulator function."""
        diag = Simulator(self)
        if diag.exec() != QtWidgets.QDialog.Accepted:
            diag.deleteLater()
            return

        helper.cm.pyload_disconnect()
        configrsc_file = helper.settings.value("simulator/configrsc", "", str)
        procimg_file = helper.settings.value("simulator/procimg", "", str)

        if helper.cm.pyload_simulate(configrsc_file, procimg_file, diag.clean_procimg):
            QtWidgets.QMessageBox.information(
                self, self.tr("Simulator started..."), self.tr(
                    "The simulator is running!\n\nYou can work with this simulator if your call "
                    "RevPiModIO with this additional parameters:\nprocimg={0}\nconfigrsc={1}\n\n"
                    "You can copy that from header textbox."
                ).format(procimg_file, configrsc_file)
            )
        else:
            pi.logger.error("Can not start simulator")
            QtWidgets.QMessageBox.critical(
                self, self.tr("Can not start..."), self.tr(
                    "Can not start the simulator! Maybe the piCtory file is corrupt "
                    "or you have no write permissions for '{0}'."
                ).format(procimg_file)
            )

        diag.deleteLater()

    @QtCore.pyqtSlot()
    def on_act_logs_triggered(self):
        """Show log window."""
        if not helper.cm.connected:
            return

        if "load_plclog" not in helper.cm.xml_funcs:
            QtWidgets.QMessageBox.warning(
                self, self.tr("Warning"), self.tr(
                    "This version of Logviewer ist not supported in version {0} "
                    "of RevPiPyLoad on your RevPi! You need at least version 0.4.1."
                ).format(helper.cm.call_remote_function("version", default_value="-"))
            )
            return None

        if self.win_log.isHidden():
            self.win_log.show()
        else:
            self.win_log.activateWindow()

    @QtCore.pyqtSlot()
    def on_act_options_triggered(self):
        """Show option dialog."""
        if not helper.cm.connected:
            return

        if helper.cm.xml_mode < 2:
            QtWidgets.QMessageBox.warning(
                self, self.tr("Warning"), self.tr(
                    "XML-RPC access mode in the RevPiPyLoad "
                    "configuration is too small to access this dialog!"
                )
            )
            return

        # Check version of RevPiPyLoad, must be greater than 0.5!
        if helper.cm.pyload_version < (0, 6, 0):
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "The Version of RevPiPyLoad on your Revolution Pi ({0}) is to old. "
                    "This Version of RevPiCommander require at least version 0.6.0 "
                    "of RevPiPyLoad. Please update your Revolution Pi!"
                )
            )
            return

        rc = self.diag_options.exec()
        if rc == QtWidgets.QDialog.Accepted:
            helper.cm.refresh_xml_mode()

    @QtCore.pyqtSlot()
    def on_act_program_triggered(self):
        """Show program dialog."""
        if not helper.cm.connected:
            return

        if helper.cm.xml_mode < 2:
            QtWidgets.QMessageBox.warning(
                self, self.tr("Warning"), self.tr(
                    "XML-RPC access mode in the RevPiPyLoad "
                    "configuration is too small to access this dialog!"
                )
            )
            return

        self.diag_program.exec()

    @QtCore.pyqtSlot()
    def on_act_developer_triggered(self):
        """Extent developer mode to main window."""
        if not helper.cm.connected:
            return

        if self.win_files.isHidden():
            self.win_files.show()
        else:
            self.win_files.activateWindow()

    @QtCore.pyqtSlot()
    def on_act_pictory_triggered(self):
        """Open piCtory in default browser of operating system."""
        if helper.cm.settings.address:
            webbrowser.open("http://{0}/".format(helper.cm.settings.address))

    @QtCore.pyqtSlot()
    def on_act_reset_triggered(self):
        """Reset piControl driver on revolution pi."""
        if not (helper.cm.connected and helper.cm.xml_mode >= 3):
            return

        ask = QtWidgets.QMessageBox.question(
            self, self.tr("Question"), self.tr(
                "Are you sure to reset piControl?\n"
                "The pictory configuration will be reloaded. During that time "
                "the process image will be interrupted and could rise errors "
                "on running control programs!"
            )
        )
        if ask != QtWidgets.QMessageBox.Yes:
            return

        ec = helper.cm.call_remote_function("resetpicontrol", default_value=-1)
        if ec == 0:
            QtWidgets.QMessageBox.information(
                self, self.tr("Success"), self.tr(
                    "piControl reset executed successfully"
                )
            )

        else:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "piControl reset could not be executed successfully"
                )
            )

    @QtCore.pyqtSlot()
    def on_act_disconnect_triggered(self):
        """Close actual connection."""
        helper.cm.pyload_disconnect()

    @QtCore.pyqtSlot(QtWidgets.QAction)
    def on_men_connections_triggered(self, action: QtWidgets.QAction):
        """A connection is selected in the men_connections menu."""
        self._pyload_connect(action.data())

    @QtCore.pyqtSlot()
    def on_act_webpage_triggered(self):
        """Open project page in default browser of operating system."""
        webbrowser.open("https://revpimodio.org")

    @QtCore.pyqtSlot()
    def on_act_info_triggered(self):
        """Open info window of application."""
        self.diag_info.exec()

    # endregion # # # # #

    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    # region #      REGION: Button events

    @QtCore.pyqtSlot()
    def on_btn_plc_start_clicked(self):
        """Start plc program on revolution pi."""
        helper.cm.call_remote_function("plcstart")

    @QtCore.pyqtSlot()
    def on_btn_plc_stop_clicked(self):
        """Start plc program on revolution pi."""
        if helper.cm.simulating:
            helper.cm.pyload_disconnect()
        else:
            helper.cm.call_remote_function("plcstop")

    @QtCore.pyqtSlot()
    def on_btn_plc_restart_clicked(self):
        """Restart plc program on revolution pi."""
        if helper.cm.simulating:
            rc = QtWidgets.QMessageBox.question(
                self, self.tr("Reset to piCtory defaults..."), self.tr(
                    "Do you want to reset your process image to {0} values?\n"
                    "You have to stop other RevPiModIO programs before doing that, "
                    "because they could reset the outputs."
                ).format(
                    self.tr("zero") if helper.settings.value("simulator/restart_zero", False, bool)
                    else self.tr("piCtory default"))
            ) == QtWidgets.QMessageBox.Yes
            if rc:
                # Set piCtory default values in process image
                helper.cm.reset_simulator()
        else:
            helper.cm.call_remote_function("plcstop")
            helper.cm.call_remote_function("plcstart")

    @QtCore.pyqtSlot(bool)
    def on_btn_plc_debug_toggled(self, state: bool):
        """Start plc watch mode."""
        if not (state or self.wid_debugcontrol is None):
            # Remove widget
            self.gl.removeWidget(self.wid_debugcontrol)
            self.wid_debugcontrol.deleteLater()
            self.wid_debugcontrol = None

        elif "psstart" not in helper.cm.xml_funcs:
            QtWidgets.QMessageBox.warning(
                self, self.tr("Warning"), self.tr(
                    "The watch mode ist not supported in version {0} "
                    "of RevPiPyLoad on your RevPi! You need at least version "
                    "0.5.3! Maybe the python3-revpimodio2 module is not "
                    "installed on your RevPi at least version 2.0.0."
                ).format(helper.cm.call_remote_function("version", "-"))
            )
            self.btn_plc_debug.setChecked(False)
            self.btn_plc_debug.setEnabled(False)

        elif helper.cm.xml_mode < 1 and not helper.cm.simulating:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Can not load this function, because your ACL level is to low!\n"
                    "You need at least level 1 to read or level 3 to write."
                )
            )
            self.btn_plc_debug.setChecked(False)

        elif helper.cm.connected or helper.cm.simulating:
            debugcontrol = DebugControl(self.centralwidget)
            if debugcontrol.reload_devices():
                self.wid_debugcontrol = debugcontrol
                self.gl.addWidget(self.wid_debugcontrol)
            else:
                debugcontrol.deleteLater()
                QtWidgets.QMessageBox.critical(
                    self, self.tr("Error"), self.tr(
                        "Can not load piCtory configuration. \n"
                        "Did you create a hardware configuration? "
                        "Please check this in piCtory!"
                    )
                )
                self.btn_plc_debug.setChecked(False)

        self.centralwidget.adjustSize()
        self.adjustSize()

    # endregion # # # # #


def main() -> int:
    """Entry point for RevPiCommander."""
    from sys import argv

    if hasattr(QtCore.Qt, 'AA_EnableHighDpiScaling'):
        QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
    if hasattr(QtCore.Qt, 'AA_UseHighDpiPixmaps'):
        QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)

    app = QtWidgets.QApplication(argv)

    try:
        # Setup translation from file with system language
        translator = QtCore.QTranslator()
        translator.load(
            QtCore.QLocale.system(),
            "revpicommander", "_", join(dirname(__file__), "locale"), ".qm"
        )
        app.installTranslator(translator)
    except Exception:
        pass

    # Prepare workers
    helper.cm.start()

    win = RevPiCommander()
    win.show()
    exit_code = app.exec()

    # Clean up workers
    helper.cm.requestInterruption()
    helper.cm.wait()

    return exit_code


if __name__ == "__main__":
    import sys

    sys.exit(main())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'backgroundworker.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_diag_backgroundworker(object):
    def setupUi(self, diag_backgroundworker):
        diag_backgroundworker.setObjectName("diag_backgroundworker")
        diag_backgroundworker.resize(424, 104)
        diag_backgroundworker.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(diag_backgroundworker)
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl_status = QtWidgets.QLabel(diag_backgroundworker)
        self.lbl_status.setText("Status message...")
        self.lbl_status.setObjectName("lbl_status")
        self.verticalLayout.addWidget(self.lbl_status)
        self.pgb_status = QtWidgets.QProgressBar(diag_backgroundworker)
        self.pgb_status.setMinimumSize(QtCore.QSize(400, 0))
        self.pgb_status.setObjectName("pgb_status")
        self.verticalLayout.addWidget(self.pgb_status)
        self.btn_box = QtWidgets.QDialogButtonBox(diag_backgroundworker)
        self.btn_box.setOrientation(QtCore.Qt.Horizontal)
        self.btn_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel)
        self.btn_box.setCenterButtons(True)
        self.btn_box.setObjectName("btn_box")
        self.verticalLayout.addWidget(self.btn_box)

        self.retranslateUi(diag_backgroundworker)
        QtCore.QMetaObject.connectSlotsByName(diag_backgroundworker)

    def retranslateUi(self, diag_backgroundworker):
        pass


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    diag_backgroundworker = QtWidgets.QDialog()
    ui = Ui_diag_backgroundworker()
    ui.setupUi(diag_backgroundworker)
    diag_backgroundworker.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'revpioption.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_diag_options(object):
    def setupUi(self, diag_options):
        diag_options.setObjectName("diag_options")
        diag_options.resize(458, 557)
        self.verticalLayout = QtWidgets.QVBoxLayout(diag_options)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gb_plc = QtWidgets.QGroupBox(diag_options)
        self.gb_plc.setObjectName("gb_plc")
        self.gridLayout = QtWidgets.QGridLayout(self.gb_plc)
        self.gridLayout.setObjectName("gridLayout")
        self.lbl_replace_io = QtWidgets.QLabel(self.gb_plc)
        self.lbl_replace_io.setObjectName("lbl_replace_io")
        self.gridLayout.addWidget(self.lbl_replace_io, 6, 0, 1, 1)
        self.cbx_zeroonerror = QtWidgets.QCheckBox(self.gb_plc)
        self.cbx_zeroonerror.setObjectName("cbx_zeroonerror")
        self.gridLayout.addWidget(self.cbx_zeroonerror, 5, 1, 1, 2)
        self.cbx_autostart = QtWidgets.QCheckBox(self.gb_plc)
        self.cbx_autostart.setObjectName("cbx_autostart")
        self.gridLayout.addWidget(self.cbx_autostart, 0, 0, 1, 3)
        self.cbb_reset_driver_action = QtWidgets.QComboBox(self.gb_plc)
        self.cbb_reset_driver_action.setObjectName("cbb_reset_driver_action")
        self.cbb_reset_driver_action.addItem("")
        self.cbb_reset_driver_action.addItem("")
        self.cbb_reset_driver_action.addItem("")
        self.gridLayout.addWidget(self.cbb_reset_driver_action, 8, 1, 1, 2)
        self.lbl_reset_driver_action = QtWidgets.QLabel(self.gb_plc)
        self.lbl_reset_driver_action.setObjectName("lbl_reset_driver_action")
        self.gridLayout.addWidget(self.lbl_reset_driver_action, 8, 0, 1, 1)
        self.lbl_plc_zero = QtWidgets.QLabel(self.gb_plc)
        self.lbl_plc_zero.setObjectName("lbl_plc_zero")
        self.gridLayout.addWidget(self.lbl_plc_zero, 3, 0, 1, 3)
        self.cbb_replace_io = QtWidgets.QComboBox(self.gb_plc)
        self.cbb_replace_io.setObjectName("cbb_replace_io")
        self.cbb_replace_io.addItem("")
        self.cbb_replace_io.addItem("")
        self.cbb_replace_io.addItem("")
        self.cbb_replace_io.addItem("")
        self.gridLayout.addWidget(self.cbb_replace_io, 6, 1, 1, 2)
        self.cbx_zeroonexit = QtWidgets.QCheckBox(self.gb_plc)
        self.cbx_zeroonexit.setObjectName("cbx_zeroonexit")
        self.gridLayout.addWidget(self.cbx_zeroonexit, 4, 1, 1, 2)
        self.lbl_plc_delay = QtWidgets.QLabel(self.gb_plc)
        self.lbl_plc_delay.setObjectName("lbl_plc_delay")
        self.gridLayout.addWidget(self.lbl_plc_delay, 2, 0, 1, 2)
        self.sbx_autoreloaddelay = QtWidgets.QSpinBox(self.gb_plc)
        self.sbx_autoreloaddelay.setMinimum(5)
        self.sbx_autoreloaddelay.setMaximum(120)
        self.sbx_autoreloaddelay.setObjectName("sbx_autoreloaddelay")
        self.gridLayout.addWidget(self.sbx_autoreloaddelay, 2, 2, 1, 1)
        self.cbx_autoreload = QtWidgets.QCheckBox(self.gb_plc)
        self.cbx_autoreload.setObjectName("cbx_autoreload")
        self.gridLayout.addWidget(self.cbx_autoreload, 1, 0, 1, 3)
        self.txt_replace_io = QtWidgets.QLineEdit(self.gb_plc)
        self.txt_replace_io.setObjectName("txt_replace_io")
        self.gridLayout.addWidget(self.txt_replace_io, 7, 1, 1, 2)
        self.lbl_lbl_reset_driver_action = QtWidgets.QLabel(self.gb_plc)
        self.lbl_lbl_reset_driver_action.setObjectName("lbl_lbl_reset_driver_action")
        self.gridLayout.addWidget(self.lbl_lbl_reset_driver_action, 9, 0, 1, 3)
        self.verticalLayout.addWidget(self.gb_plc)
        self.gb_server = QtWidgets.QGroupBox(diag_options)
        self.gb_server.setObjectName("gb_server")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.gb_server)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.btn_aclplcserver = QtWidgets.QPushButton(self.gb_server)
        self.btn_aclplcserver.setObjectName("btn_aclplcserver")
        self.gridLayout_2.addWidget(self.btn_aclplcserver, 0, 1, 1, 1)
        self.cbx_mqtt = QtWidgets.QCheckBox(self.gb_server)
        self.cbx_mqtt.setObjectName("cbx_mqtt")
        self.gridLayout_2.addWidget(self.cbx_mqtt, 2, 0, 1, 1)
        self.cbx_plcserver = QtWidgets.QCheckBox(self.gb_server)
        self.cbx_plcserver.setObjectName("cbx_plcserver")
        self.gridLayout_2.addWidget(self.cbx_plcserver, 0, 0, 1, 1)
        self.lbl_server_status = QtWidgets.QLabel(self.gb_server)
        self.lbl_server_status.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl_server_status.setObjectName("lbl_server_status")
        self.gridLayout_2.addWidget(self.lbl_server_status, 1, 1, 1, 1)
        self.lbl_lbl_server_status = QtWidgets.QLabel(self.gb_server)
        self.lbl_lbl_server_status.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lbl_lbl_server_status.setObjectName("lbl_lbl_server_status")
        self.gridLayout_2.addWidget(self.lbl_lbl_server_status, 1, 0, 1, 1)
        self.lbl_mqtt_status = QtWidgets.QLabel(self.gb_server)
        self.lbl_mqtt_status.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl_mqtt_status.setObjectName("lbl_mqtt_status")
        self.gridLayout_2.addWidget(self.lbl_mqtt_status, 3, 1, 1, 1)
        self.lbl_lbl_mqtt_status = QtWidgets.QLabel(self.gb_server)
        self.lbl_lbl_mqtt_status.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lbl_lbl_mqtt_status.setObjectName("lbl_lbl_mqtt_status")
        self.gridLayout_2.addWidget(self.lbl_lbl_mqtt_status, 3, 0, 1, 1)
        self.btn_mqtt = QtWidgets.QPushButton(self.gb_server)
        self.btn_mqtt.setObjectName("btn_mqtt")
        self.gridLayout_2.addWidget(self.btn_mqtt, 2, 1, 1, 1)
        self.btn_aclxmlrpc = QtWidgets.QPushButton(self.gb_server)
        self.btn_aclxmlrpc.setObjectName("btn_aclxmlrpc")
        self.gridLayout_2.addWidget(self.btn_aclxmlrpc, 4, 1, 1, 1)
        self.cbx_xmlrpc = QtWidgets.QCheckBox(self.gb_server)
        self.cbx_xmlrpc.setObjectName("cbx_xmlrpc")
        self.gridLayout_2.addWidget(self.cbx_xmlrpc, 4, 0, 1, 1)
        self.verticalLayout.addWidget(self.gb_server)
        self.btn_box = QtWidgets.QDialogButtonBox(diag_options)
        self.btn_box.setOrientation(QtCore.Qt.Horizontal)
        self.btn_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.btn_box.setObjectName("btn_box")
        self.verticalLayout.addWidget(self.btn_box)

        self.retranslateUi(diag_options)
        self.btn_box.accepted.connect(diag_options.accept) # type: ignore
        self.btn_box.rejected.connect(diag_options.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(diag_options)

    def retranslateUi(self, diag_options):
        _translate = QtCore.QCoreApplication.translate
        diag_options.setWindowTitle(_translate("diag_options", "RevPi Python PLC Options"))
        self.gb_plc.setTitle(_translate("diag_options", "Start / Stop behavior of PLC program"))
        self.lbl_replace_io.setText(_translate("diag_options", "Replace IO file:"))
        self.cbx_zeroonerror.setText(_translate("diag_options", "... after exception and errors"))
        self.cbx_autostart.setText(_translate("diag_options", "Start PLC program automatically"))
        self.cbb_reset_driver_action.setItemText(0, _translate("diag_options", "Do nothing"))
        self.cbb_reset_driver_action.setItemText(1, _translate("diag_options", "Restart after piCtory changed"))
        self.cbb_reset_driver_action.setItemText(2, _translate("diag_options", "Always restart the PLC program"))
        self.lbl_reset_driver_action.setText(_translate("diag_options", "Driver reset action:"))
        self.lbl_plc_zero.setText(_translate("diag_options", "Set process image to NULL if program terminates..."))
        self.cbb_replace_io.setItemText(0, _translate("diag_options", "Do not use replace io file"))
        self.cbb_replace_io.setItemText(1, _translate("diag_options", "Use static file from RevPiPyLoad"))
        self.cbb_replace_io.setItemText(2, _translate("diag_options", "Use dynamic file from work directory"))
        self.cbb_replace_io.setItemText(3, _translate("diag_options", "Give own path and filename"))
        self.cbx_zeroonexit.setText(_translate("diag_options", "... sucessfully without error"))
        self.lbl_plc_delay.setText(_translate("diag_options", "Restart delay in seconds:"))
        self.cbx_autoreload.setText(_translate("diag_options", "Restart PLC program after exit or crash"))
        self.lbl_lbl_reset_driver_action.setText(_translate("diag_options", "PLC program behavior after piCtory driver reset clicked"))
        self.gb_server.setTitle(_translate("diag_options", "RevPiPyLoad server services"))
        self.btn_aclplcserver.setText(_translate("diag_options", "Edit ACL"))
        self.cbx_mqtt.setText(_translate("diag_options", "MQTT process image publisher"))
        self.cbx_plcserver.setText(_translate("diag_options", "Start RevPi piControl server"))
        self.lbl_server_status.setText(_translate("diag_options", "status"))
        self.lbl_lbl_server_status.setText(_translate("diag_options", "piControl server is:"))
        self.lbl_mqtt_status.setText(_translate("diag_options", "status"))
        self.lbl_lbl_mqtt_status.setText(_translate("diag_options", "MQTT publish service is:"))
        self.btn_mqtt.setText(_translate("diag_options", "Settings"))
        self.btn_aclxmlrpc.setText(_translate("diag_options", "Edit ACL"))
        self.cbx_xmlrpc.setText(_translate("diag_options", "Activate XML-RPC for RevPiCommander"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    diag_options = QtWidgets.QDialog()
    ui = Ui_diag_options()
    ui.setupUi(diag_options)
    diag_options.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'debugcontrol.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wid_debugcontrol(object):
    def setupUi(self, wid_debugcontrol):
        wid_debugcontrol.setObjectName("wid_debugcontrol")
        wid_debugcontrol.resize(402, 214)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(wid_debugcontrol.sizePolicy().hasHeightForWidth())
        wid_debugcontrol.setSizePolicy(sizePolicy)
        self.gridLayout = QtWidgets.QGridLayout(wid_debugcontrol)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.gb_devices = QtWidgets.QGroupBox(wid_debugcontrol)
        self.gb_devices.setObjectName("gb_devices")
        self.vl_devices = QtWidgets.QVBoxLayout(self.gb_devices)
        self.vl_devices.setObjectName("vl_devices")
        self.gridLayout.addWidget(self.gb_devices, 0, 0, 1, 1)
        self.cbx_stay_on_top = QtWidgets.QCheckBox(wid_debugcontrol)
        self.cbx_stay_on_top.setObjectName("cbx_stay_on_top")
        self.gridLayout.addWidget(self.cbx_stay_on_top, 1, 0, 1, 1)
        self.gb_control = QtWidgets.QGroupBox(wid_debugcontrol)
        self.gb_control.setObjectName("gb_control")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.gb_control)
        self.verticalLayout.setObjectName("verticalLayout")
        self.btn_read_io = QtWidgets.QPushButton(self.gb_control)
        self.btn_read_io.setAutoRepeat(True)
        self.btn_read_io.setAutoRepeatDelay(100)
        self.btn_read_io.setAutoRepeatInterval(200)
        self.btn_read_io.setObjectName("btn_read_io")
        self.verticalLayout.addWidget(self.btn_read_io)
        self.btn_refresh_io = QtWidgets.QPushButton(self.gb_control)
        self.btn_refresh_io.setAutoRepeat(True)
        self.btn_refresh_io.setAutoRepeatDelay(100)
        self.btn_refresh_io.setAutoRepeatInterval(200)
        self.btn_refresh_io.setObjectName("btn_refresh_io")
        self.verticalLayout.addWidget(self.btn_refresh_io)
        self.btn_write_o = QtWidgets.QPushButton(self.gb_control)
        self.btn_write_o.setObjectName("btn_write_o")
        self.verticalLayout.addWidget(self.btn_write_o)
        self.cbx_refresh = QtWidgets.QCheckBox(self.gb_control)
        self.cbx_refresh.setObjectName("cbx_refresh")
        self.verticalLayout.addWidget(self.cbx_refresh)
        self.cbx_write = QtWidgets.QCheckBox(self.gb_control)
        self.cbx_write.setObjectName("cbx_write")
        self.verticalLayout.addWidget(self.cbx_write)
        spacerItem = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.gridLayout.addWidget(self.gb_control, 0, 1, 2, 1)

        self.retranslateUi(wid_debugcontrol)
        QtCore.QMetaObject.connectSlotsByName(wid_debugcontrol)

    def retranslateUi(self, wid_debugcontrol):
        _translate = QtCore.QCoreApplication.translate
        self.gb_devices.setTitle(_translate("wid_debugcontrol", "Revolution Pi devices"))
        self.cbx_stay_on_top.setText(_translate("wid_debugcontrol", "Open to stay on top"))
        self.gb_control.setTitle(_translate("wid_debugcontrol", "IO Control"))
        self.btn_read_io.setToolTip(_translate("wid_debugcontrol", "Read all IO values and discard local changes (F4)\n"
"\n"
"Hold this button pressed and it will refresh the IOs every 200 ms."))
        self.btn_read_io.setText(_translate("wid_debugcontrol", "Read &all IO values"))
        self.btn_refresh_io.setToolTip(_translate("wid_debugcontrol", "Refresh all IO values which are locally not changed (F5)\n"
"\n"
"Hold this button pressed and it will refresh the IOs every 200 ms."))
        self.btn_refresh_io.setText(_translate("wid_debugcontrol", "&Refresh unchanged IOs"))
        self.btn_write_o.setToolTip(_translate("wid_debugcontrol", "Write locally changed output values to process image (F6)"))
        self.btn_write_o.setText(_translate("wid_debugcontrol", "&Write changed outputs"))
        self.cbx_refresh.setText(_translate("wid_debugcontrol", "&Auto refresh values"))
        self.cbx_write.setText(_translate("wid_debugcontrol", "and write outputs"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    wid_debugcontrol = QtWidgets.QWidget()
    ui = Ui_wid_debugcontrol()
    ui.setupUi(wid_debugcontrol)
    wid_debugcontrol.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""Saved connections of Revolution Pi devices."""
__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2023 Sven Sager"
__license__ = "GPLv2"

from enum import IntEnum

import keyring
from PyQt5 import QtCore, QtGui, QtWidgets
from keyring.errors import KeyringError

from . import helper
from . import proginit as pi
from .helper import RevPiSettings, WidgetData
from .ui.revpiplclist_ui import Ui_diag_connections


class NodeType(IntEnum):
    CON = 1000
    DIR = 1001


class RevPiPlcList(QtWidgets.QDialog, Ui_diag_connections):
    """Manage your saved connections."""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.__default_port = 55123

        self.__current_item = QtWidgets.QTreeWidgetItem()  # type: QtWidgets.QTreeWidgetItem
        self.changes = True
        self._keyring_cleanup_id_user = []

        self.tre_connections.setColumnWidth(0, 250)
        self.lbl_port.setText(self.lbl_port.text().format(self.__default_port))
        self.sbx_port.setValue(self.__default_port)

        # Dirty workaround to remove default button to prevent action on ENTER key, while user edit texts
        self.__btn_dummy = QtWidgets.QPushButton(self)
        self.__btn_dummy.setVisible(False)
        self.__btn_dummy.setDefault(True)

    def _load_cbb_folder(self):
        """Clean up all entries and reload existing ones from treeview."""
        self.cbb_folder.blockSignals(True)

        self.cbb_folder.clear()
        self.cbb_folder.addItem("")
        for i in range(self.tre_connections.topLevelItemCount()):
            item = self.tre_connections.topLevelItem(i)
            if item.type() != NodeType.DIR:
                continue
            self.cbb_folder.addItem(item.text(0))

        self.cbb_folder.blockSignals(False)

    def _load_settings(self):
        """Load values to GUI widgets."""
        pi.logger.debug("RevPiPlcList._load_settings")

        self.tre_connections.clear()

        # Get length of array and close it, the RevPiSettings-class need it
        count_settings = helper.settings.beginReadArray("connections")
        helper.settings.endArray()

        for i in range(count_settings):
            settings = RevPiSettings(i)

            con_item = QtWidgets.QTreeWidgetItem(NodeType.CON)
            con_item.setIcon(0, QtGui.QIcon(":/main/ico/cpu.ico"))
            con_item.setText(0, settings.name)
            con_item.setText(1, settings.address)

            con_item.setData(0, WidgetData.revpi_settings, settings)

            folder = settings.folder
            if folder:
                sub_folder = self._get_folder_item(folder)
                if sub_folder is None:
                    sub_folder = QtWidgets.QTreeWidgetItem(NodeType.DIR)
                    sub_folder.setIcon(0, QtGui.QIcon(":/main/ico/folder.ico"))
                    sub_folder.setText(0, folder)
                    self.tre_connections.addTopLevelItem(sub_folder)

                sub_folder.addChild(con_item)
            else:
                self.tre_connections.addTopLevelItem(con_item)

        self.tre_connections.expandAll()
        self.changes = False

        self._edit_state()

    def accept(self) -> None:
        pi.logger.debug("RevPiPlcList.accept")

        for internal_id, ssh_user in self._keyring_cleanup_id_user:
            service_name = "{0}.{1}_{2}".format(
                helper.settings.applicationName(),
                helper.settings.organizationName(),
                internal_id
            )
            try:
                # Remove information from os keyring, which we collected on_btn_delete_clicked
                keyring.delete_password(service_name, ssh_user)
            except KeyringError as e:
                pi.logger.error(e)

        helper.settings.remove("connections")

        for i in range(self.tre_connections.topLevelItemCount()):
            root_item = self.tre_connections.topLevelItem(i)
            if root_item.type() == NodeType.DIR:
                for k in range(root_item.childCount()):
                    revpi_settings = root_item.child(k).data(0, WidgetData.revpi_settings)  # type: RevPiSettings
                    revpi_settings.folder = root_item.text(0)
                    revpi_settings.save_settings()
            elif root_item.type() == NodeType.CON:
                revpi_settings = root_item.data(0, WidgetData.revpi_settings)  # type: RevPiSettings
                revpi_settings.folder = ""
                revpi_settings.save_settings()

        self.changes = False
        super().accept()

    def closeEvent(self, a0: QtGui.QCloseEvent) -> None:
        pi.logger.debug("RevPiPlcList.closeEvent")
        if self.changes:
            ask = QtWidgets.QMessageBox.question(
                self, self.tr("Question"), self.tr(
                    "Do you really want to quit? \nUnsaved changes will be lost."
                )
            ) == QtWidgets.QMessageBox.Yes

            if ask:
                self.reject()
            else:
                a0.ignore()

    def exec(self) -> int:
        self._load_settings()
        return super().exec()

    def exec_with_presets(self, presets: RevPiSettings) -> int:
        """
        Start dialog with new created settings object and presets.

        :param presets: Use these settings as preset
        :return: Dialog status
        """
        self._load_settings()
        self.on_btn_add_clicked(presets)
        return super().exec()

    @QtCore.pyqtSlot(QtWidgets.QAbstractButton)
    def on_btn_box_clicked(self, button: QtWidgets.QAbstractButton):
        if self.btn_box.buttonRole(button) == QtWidgets.QDialogButtonBox.DestructiveRole:
            self.reject()

    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    # region #      REGION: Connection management

    def _edit_state(self):
        """Set enabled status of all controls, depending on selected item."""
        pi.logger.debug("RevPiPlcList._edit_state")

        item = self.tre_connections.currentItem()
        if item is None:
            up_ok = False
            down_ok = False
            con_item = False
            dir_item = False
        else:
            con_item = item.type() == NodeType.CON
            dir_item = item.type() == NodeType.DIR

            if item.parent():
                index = item.parent().indexOfChild(item)
                up_ok = index > 0
                down_ok = index < item.parent().childCount() - 1
            else:
                index = self.tre_connections.indexOfTopLevelItem(item)
                up_ok = index > 0
                down_ok = index < self.tre_connections.topLevelItemCount() - 1

        self.btn_up.setEnabled(up_ok)
        self.btn_down.setEnabled(down_ok)
        self.btn_delete.setEnabled(con_item or dir_item)
        self.txt_name.setEnabled(con_item)
        self.txt_address.setEnabled(con_item)
        self.sbx_port.setEnabled(con_item)
        self.sbx_timeout.setEnabled(con_item)
        self.cbb_folder.setEnabled(con_item or dir_item)
        self.cbb_folder.setEditable(dir_item)
        if self.cbb_folder.isEditable():
            # Disable auto complete, this would override a new typed name with existing one
            self.cbb_folder.setCompleter(None)

        self.cbx_ssh_use_tunnel.setEnabled(con_item)
        self.sbx_ssh_port.setEnabled(con_item)
        self.txt_ssh_user.setEnabled(con_item)

    def _get_folder_item(self, name: str):
        """Find the folder entry by name."""
        for i in range(self.tre_connections.topLevelItemCount()):
            tli = self.tre_connections.topLevelItem(i)
            if tli.type() == NodeType.DIR and tli.text(0) == name:
                return tli
        return None

    def _move_item(self, count: int):
        """Move connection item up or down"""
        item = self.tre_connections.currentItem()
        if not item:
            return

        if item.parent():
            dir_item = item.parent()
            index = dir_item.indexOfChild(item)
            new_index = index + count
            if 0 <= new_index < dir_item.childCount():
                item = dir_item.takeChild(index)
                dir_item.insertChild(new_index, item)
        else:
            index = self.tre_connections.indexOfTopLevelItem(item)
            new_index = index + count
            if 0 <= index < self.tre_connections.topLevelItemCount():
                item = self.tre_connections.takeTopLevelItem(index)
                self.tre_connections.insertTopLevelItem(new_index, item)
                if item.type() == NodeType.DIR:
                    # Expand a moved dir node, it would be collapsed after move
                    self.tre_connections.expandItem(item)

        self.tre_connections.setCurrentItem(item)

    @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem, QtWidgets.QTreeWidgetItem)
    def on_tre_connections_currentItemChanged(
            self, current: QtWidgets.QTreeWidgetItem, previous: QtWidgets.QTreeWidgetItem):

        self._edit_state()
        self._load_cbb_folder()

        if current and current.type() == NodeType.CON:
            self.__current_item = current

            settings = current.data(0, WidgetData.revpi_settings)  # type: RevPiSettings
            self.txt_name.setText(settings.name)
            self.txt_address.setText(settings.address)
            self.sbx_port.setValue(settings.port)
            self.sbx_timeout.setValue(settings.timeout)
            if current.parent() is None:
                self.cbb_folder.setCurrentIndex(0)
            else:
                self.cbb_folder.setCurrentText(current.parent().text(0))

            self.cbx_ssh_use_tunnel.setChecked(settings.ssh_use_tunnel)
            self.sbx_ssh_port.setValue(settings.ssh_port)
            self.txt_ssh_user.setText(settings.ssh_user)

        elif current and current.type() == NodeType.DIR:
            self.__current_item = current
            self.cbb_folder.setCurrentText(current.text(0))

        else:
            self.__current_item = QtWidgets.QTreeWidgetItem()
            self.cbb_folder.setCurrentText(current.text(0) if current else "")

    @QtCore.pyqtSlot()
    def on_btn_up_clicked(self):
        self._move_item(-1)

    @QtCore.pyqtSlot()
    def on_btn_down_clicked(self):
        self._move_item(1)

    @QtCore.pyqtSlot()
    def on_btn_delete_clicked(self):
        """Remove selected entry."""

        def remove_item(item: QtWidgets.QTreeWidgetItem):
            """Remove CON item and save keyring actions for save action."""
            revpi_settings = item.data(0, WidgetData.revpi_settings)  # type: RevPiSettings
            if revpi_settings.ssh_saved_password:
                # Cleans up keyring in save function
                self._keyring_cleanup_id_user.append((revpi_settings.internal_id, revpi_settings.ssh_user))

            dir_node = item.parent()
            if dir_node:
                dir_node.removeChild(item)
            else:
                index = self.tre_connections.indexOfTopLevelItem(item)
                self.tre_connections.takeTopLevelItem(index)

        item_to_remove = self.tre_connections.currentItem()

        if item_to_remove and item_to_remove.type() == NodeType.DIR:
            if item_to_remove.childCount():
                rc = QtWidgets.QMessageBox.question(
                    self, self.tr("Question"), self.tr(
                        "If you remote this folder, all containing elements will be removed, too. \n\n"
                        "Do you want to delete folder and all elements?"
                    ),
                )
                if rc != QtWidgets.QMessageBox.Yes:
                    return

                while item_to_remove.childCount() > 0:
                    remove_item(item_to_remove.child(0))

            item_index = self.tre_connections.indexOfTopLevelItem(item_to_remove)
            self.tre_connections.takeTopLevelItem(item_index)

        elif item_to_remove and item_to_remove.type() == NodeType.CON:
            remove_item(item_to_remove)

    @QtCore.pyqtSlot()
    def on_btn_add_clicked(self, settings_preset: RevPiSettings = None):
        """Create new element."""
        settings = settings_preset or RevPiSettings()
        new_item = QtWidgets.QTreeWidgetItem(NodeType.CON)
        new_item.setIcon(0, QtGui.QIcon(":/main/ico/cpu.ico"))
        new_item.setText(0, settings.name)
        new_item.setData(0, WidgetData.revpi_settings, settings)
        sub_folder = self._get_folder_item(self.cbb_folder.currentText())
        if sub_folder:
            sub_folder.addChild(new_item)
        else:
            self.tre_connections.addTopLevelItem(new_item)

        # This will load all settings and prepare widgets
        self.tre_connections.setCurrentItem(new_item)
        self.txt_name.setFocus()
        self.txt_name.selectAll()

    @QtCore.pyqtSlot()
    def on_btn_add_dir_clicked(self):
        """Add a new folder."""
        folder_text = self.tr("New folder")
        sub_folder = QtWidgets.QTreeWidgetItem(NodeType.DIR)
        sub_folder.setIcon(0, QtGui.QIcon(":/main/ico/folder.ico"))
        sub_folder.setText(0, folder_text)

        self.tre_connections.addTopLevelItem(sub_folder)
        self.tre_connections.setCurrentItem(sub_folder)
        self.cbb_folder.setFocus()

        self.changes = True

    @QtCore.pyqtSlot(str)
    def on_txt_name_textEdited(self, text):
        if self.__current_item.type() != NodeType.CON:
            return
        self.__current_item.setText(0, text)
        settings = self.__current_item.data(0, WidgetData.revpi_settings)  # type: RevPiSettings
        settings.name = text
        self.changes = True

    @QtCore.pyqtSlot(str)
    def on_txt_address_textEdited(self, text):
        if self.__current_item.type() != NodeType.CON:
            return
        self.__current_item.setText(1, text)
        settings = self.__current_item.data(0, WidgetData.revpi_settings)  # type: RevPiSettings
        settings.address = text
        self.changes = True

    @QtCore.pyqtSlot(int)
    def on_sbx_port_valueChanged(self, value: int):
        if self.__current_item.type() != NodeType.CON:
            return
        settings = self.__current_item.data(0, WidgetData.revpi_settings)  # type: RevPiSettings
        settings.port = value
        self.changes = True

    @QtCore.pyqtSlot(int)
    def on_sbx_timeout_valueChanged(self, value: int):
        if self.__current_item.type() != NodeType.CON:
            return
        settings = self.__current_item.data(0, WidgetData.revpi_settings)  # type: RevPiSettings
        settings.timeout = value
        self.changes = True

    @QtCore.pyqtSlot(int)
    def on_cbx_ssh_use_tunnel_stateChanged(self, check_state: int):
        if self.__current_item.type() != NodeType.CON:
            return
        settings = self.__current_item.data(0, WidgetData.revpi_settings)  # type: RevPiSettings
        settings.ssh_use_tunnel = check_state == QtCore.Qt.CheckState.Checked
        self.changes = True

    @QtCore.pyqtSlot(int)
    def on_sbx_ssh_port_valueChanged(self, value: int):
        if self.__current_item.type() != NodeType.CON:
            return
        settings = self.__current_item.data(0, WidgetData.revpi_settings)  # type: RevPiSettings
        settings.ssh_port = value
        self.changes = True

    @QtCore.pyqtSlot(str)
    def on_txt_ssh_user_textEdited(self, text):
        if self.__current_item.type() != NodeType.CON:
            return
        settings = self.__current_item.data(0, WidgetData.revpi_settings)  # type: RevPiSettings
        settings.ssh_user = text
        self.changes = True

    @QtCore.pyqtSlot(str)
    def on_cbb_folder_currentIndexChanged(self, text: str):
        pi.logger.debug("RevPiPlcList.on_cbb_folder_currentIndexChanged({0})".format(text))

        if self.__current_item.type() == NodeType.CON:
            new_dir_node = self._get_folder_item(text)
            current_dir_node = self.__current_item.parent()
            if current_dir_node == new_dir_node:
                # No change required, both nodes are the same
                return

            change_item = self.__current_item
            self.tre_connections.blockSignals(True)
            self.changes = True

            if current_dir_node:
                # Move an element to other folder or root
                index = current_dir_node.indexOfChild(change_item)
                change_item = current_dir_node.takeChild(index)

            else:
                # Move a root element to a folder
                index = self.tre_connections.indexOfTopLevelItem(change_item)
                change_item = self.tre_connections.takeTopLevelItem(index)

            if text == "":
                self.tre_connections.addTopLevelItem(change_item)

            else:
                new_dir_node.addChild(change_item)

            self.tre_connections.blockSignals(False)
            self.tre_connections.setCurrentItem(change_item)

    @QtCore.pyqtSlot(str)
    def on_cbb_folder_editTextChanged(self, text: str):
        pi.logger.debug("RevPiPlcList.on_cbb_folder_editTextChanged({0})".format(text))

        if self.__current_item.type() == NodeType.DIR and self.__current_item.text(0) != text:
            # We just have to rename the dir node
            self.__current_item.setText(0, text)
            self.changes = True

    # endregion # # # # #



# -*- coding: utf-8 -*-
"""View log files from Revolution Pi."""
__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2023 Sven Sager"
__license__ = "GPLv2"

from enum import IntEnum

from PyQt5 import QtCore, QtGui, QtWidgets

from . import helper
from . import proginit as pi
from .ui.revpilogfile_ui import Ui_win_revpilogfile


class LogType(IntEnum):
    NONE = 0
    APP = 1
    DAEMON = 2


class DataThread(QtCore.QThread):
    error_detected = QtCore.pyqtSignal(str)
    line_logged = QtCore.pyqtSignal(LogType, bool, str)
    """log_type, success, text"""

    def __init__(self, parent=None, cycle_time=1000):
        super().__init__(parent)

        self._cycle_time = cycle_time
        self._paused = True
        self.error_count = 0
        self.max_block = 16384  # 16 kByte
        self.mrk_app = 0
        self.mrk_daemon = 0

    def _load_log(self, log_type: LogType, xmlcall, start_position: int):
        """
        Get log text and put it to log viewer.

        :param log_type: Type of log file <class 'LogType'>
        :param xmlcall: XML-RPC call to get log text
        :param start_position: Start position in log file to read from
        :return: tuple(position: int, EOF: bool)
        """
        # Load max data from start position
        buff_log = helper.cm.call_remote_function(
            xmlcall,
            start_position,
            self.max_block,
            raise_exception=True
        ).data  # type: bytes

        eof = True
        if buff_log == b'\x16':  # 'ESC'
            # RevPiPyLoad could not access log file on Revolution Pi
            self.line_logged.emit(log_type, False, "")

        elif buff_log == b'\x19':  # 'EndOfMedia'
            # The log file was rotated by log rotate on the Revolution Pi
            start_position = 0
            eof = False
            pi.logger.info("RevPi started a new log file.")

        elif buff_log:
            start_position += len(buff_log)
            eof = len(buff_log) < self.max_block
            self.line_logged.emit(log_type, True, buff_log.decode("utf-8", errors="replace"))

        return start_position, eof

    def pause(self):
        """Stop checking new log lines, but leave thread alive."""
        pi.logger.debug("DataThread.pause")
        self._paused = True

    def resume(self):
        """Start checking for new log lines."""
        pi.logger.debug("DataThread.resume")
        self._paused = False

    def run(self) -> None:
        pi.logger.debug("DataThread.run")

        while not self.isInterruptionRequested():
            eof_app = False
            eof_daemon = False
            if not self._paused:
                try:
                    while not (eof_app or self.isInterruptionRequested()):
                        self.mrk_app, eof_app = self._load_log(
                            LogType.APP,
                            "load_applog",
                            self.mrk_app,
                        )
                    while not (eof_daemon or self.isInterruptionRequested()):
                        self.mrk_daemon, eof_daemon = self._load_log(
                            LogType.DAEMON,
                            "load_plclog",
                            self.mrk_daemon,
                        )
                    self.error_count = 0
                except Exception as e:
                    if self.error_count == 0:
                        self.error_detected.emit(str(e))
                    self.error_count += 1

            self.msleep(self._cycle_time)


class RevPiLogfile(QtWidgets.QMainWindow, Ui_win_revpilogfile):
    """Log file viewer for daemon and plc program log."""

    def __init__(self, parent=None):
        u"""Init RevPiLogfile-Class."""
        super().__init__(parent)
        self.setupUi(self)

        self.th_data = DataThread(self)
        self.err_daemon = 0

        helper.cm.connection_established.connect(self.on_cm_connection_established)
        helper.cm.connection_disconnecting.connect(self.on_cm_connection_disconnecting)

        self._load_gui_settings()

    def _create_data_thread(self):
        self.th_data.deleteLater()

        self.th_data = DataThread(self)
        self.th_data.error_detected.connect(self.txt_daemon.setPlainText)
        self.th_data.line_logged.connect(self.on_line_logged)
        self.th_data.start()

    def closeEvent(self, a0: QtGui.QCloseEvent) -> None:
        helper.settings.setValue("logfile/geo", self.saveGeometry())
        helper.settings.setValue("logfile/stay_on_top", self.cbx_stay_on_top.isChecked())

    def hideEvent(self, a0: QtGui.QHideEvent) -> None:
        self.th_data.pause()
        super().hideEvent(a0)

    def showEvent(self, a0: QtGui.QShowEvent) -> None:
        self.th_data.resume()
        super().showEvent(a0)

    def _load_gui_settings(self):
        self.restoreGeometry(helper.settings.value("logfile/geo", b''))
        self.cbx_stay_on_top.setChecked(helper.settings.value("logfile/stay_on_top", False, bool))
        self.cbx_wrap.setChecked(helper.settings.value("logfile/wrap", False, bool))

    @QtCore.pyqtSlot()
    def on_cm_connection_disconnecting(self):
        """Disconnecting form Revolution Pi."""
        self.th_data.requestInterruption()
        self.th_data.wait()

    @QtCore.pyqtSlot()
    def on_cm_connection_established(self):
        """New connection established."""
        self.txt_app.clear()
        self.txt_daemon.clear()

        self._create_data_thread()
        if self.isVisible():
            self.th_data.resume()

    @QtCore.pyqtSlot()
    def on_btn_daemon_clicked(self):
        """Clear the daemon log view."""
        self.txt_daemon.clear()

    @QtCore.pyqtSlot()
    def on_btn_app_clicked(self):
        """Clear the app log view."""
        self.txt_app.clear()

    @QtCore.pyqtSlot(int)
    def on_cbx_stay_on_top_stateChanged(self, state: int):
        """Set flag to stay on top of all windows."""
        self.setWindowFlag(QtCore.Qt.WindowStaysOnTopHint, state == QtCore.Qt.Checked)

    @QtCore.pyqtSlot(int)
    def on_cbx_wrap_stateChanged(self, state: int):
        """Line wrap mode."""
        wrap_mode = QtWidgets.QPlainTextEdit.WidgetWidth if state == QtCore.Qt.Checked else \
            QtWidgets.QPlainTextEdit.NoWrap
        self.txt_daemon.setLineWrapMode(wrap_mode)
        self.txt_app.setLineWrapMode(wrap_mode)
        helper.settings.setValue("logfile/wrap", self.cbx_wrap.isChecked())

    @QtCore.pyqtSlot(LogType, bool, str)
    def on_line_logged(self, log_type: LogType, success: bool, text: str):
        pi.logger.debug("RevPiLogfile.on_line_logged")

        if log_type == LogType.APP:
            textwidget = self.txt_app
        elif log_type == LogType.DAEMON:
            textwidget = self.txt_daemon
        else:
            return

        bar = textwidget.verticalScrollBar()
        auto_scroll = bar.value() == bar.maximum()

        if not success:
            textwidget.clear()
            textwidget.setPlainText(self.tr("Can not access log file on the RevPi"))
        elif text != "":
            # Function will add \n automatically
            textwidget.appendPlainText(text.strip("\n"))
            if auto_scroll:
                bar.setValue(bar.maximum())



# -*- coding: utf-8 -*-
"""File transfer system to handle QThreads."""
__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2023 Sven Sager"
__license__ = "GPLv2"

from logging import getLogger

from PyQt5 import QtCore, QtGui, QtWidgets

from .ui.backgroundworker_ui import Ui_diag_backgroundworker

log = getLogger()


class BackgroundWorker(QtCore.QThread):
    steps_todo = QtCore.pyqtSignal(int)
    steps_done = QtCore.pyqtSignal(int)
    status_message = QtCore.pyqtSignal(str)

    def __init__(self, parent=None, interruption_text: str = None):
        super().__init__(parent)
        self._interruption_text = interruption_text or self.tr("User requested cancellation...")

    def check_cancel(self) -> bool:
        """
        Check for interruption of thread and show message

        :return: True, if interruption was requested
        """
        if self.isInterruptionRequested():
            self.status_message.emit(self._interruption_text)
            self.msleep(750)
            return True
        return False

    def exec_dialog(self, window_title="", can_cancel=True) -> int:
        """
        Show dialog with progress bar.

        :param window_title: Title of Dialog window
        :param can_cancel: If False, the cancel button is deactivated
        :return: Dialog result
        """
        diag = WorkerDialog(self, self.parent())
        diag.setWindowTitle(window_title)
        diag.btn_box.setEnabled(can_cancel)
        rc = diag.exec()
        diag.deleteLater()
        return rc

    def wait_interruptable(self, seconds=-1) -> bool:
        """
        Save function to wait and get the cancel buttons.

        :param seconds: Wait this amount of seconds
        :return: True, if interruption was requested
        """
        counter = seconds * 4
        while counter != 0:
            counter -= 1
            self.msleep(250)
            if self.check_cancel():
                return True
        return False

    def run(self) -> None:
        """Override this function with your logic."""
        raise NotImplementedError()


class BackgroundWaiter(BackgroundWorker):
    """Just wait an amount of time and show progress bar."""

    def __init__(self, seconds: int, status_message: str, parent=None, interruption_text: str = None):
        super().__init__(parent, interruption_text)
        self._status_message = status_message
        self._wait_steps = seconds * 4

    def run(self) -> None:
        log.debug("BackgroundWaiter.run")
        self.steps_todo.emit(self._wait_steps)
        self.status_message.emit(self._status_message)
        counter = 0
        while counter <= self._wait_steps:
            counter += 1
            self.msleep(250)
            if self.isInterruptionRequested():
                self.steps_done.emit(self._wait_steps)
            if self.check_cancel():
                return
            self.steps_done.emit(counter)


class WorkerDialog(QtWidgets.QDialog, Ui_diag_backgroundworker):

    def __init__(self, worker_thread: BackgroundWorker, parent=None):
        """
        Base of dialog to show progress from a background thread.

        :param worker_thread: Thread with the logic work to do
        :param parent: QtWidget
        """
        super().__init__(parent)
        self.setupUi(self)

        self._canceled = False

        self._th = worker_thread
        self._th.finished.connect(self.on_th_finished)
        self._th.steps_todo.connect(self.pgb_status.setMaximum)
        self._th.steps_done.connect(self.pgb_status.setValue)
        self._th.status_message.connect(self.lbl_status.setText)

    def closeEvent(self, a0: QtGui.QCloseEvent) -> None:
        a0.ignore()

    def exec(self) -> int:
        self._th.start()
        return super().exec()

    @QtCore.pyqtSlot()
    def on_th_finished(self) -> None:
        """Check the result of import thread."""
        if self._canceled:
            self.reject()
        else:
            self.accept()

    @QtCore.pyqtSlot(QtWidgets.QAbstractButton)
    def on_btn_box_clicked(self, button: QtWidgets.QAbstractButton) -> None:
        """Control buttons for dialog."""
        role = self.btn_box.buttonRole(button)
        log.debug("WorkerDialog.on_btn_box_clicked({0})".format(role))

        if role == QtWidgets.QDialogButtonBox.RejectRole:
            self._th.requestInterruption()
            self._canceled = True
